function net = rbfnet(varargin)
% RBFNET Constructs an adaptive hyperplane radial basis
%   function network
%   RBFNET(nin,nhid,nout,learnp,irange);
%       nin     - number of inputs
%       nhid    - number of bases per input dimension
%       nout    - number of outputs
%       learnp  - learning rate
%       irange  - input range
%       vers    - version of getcenter to use [1,2,3]

% Satrajit Ghosh, SpeechLab, Boston University. (c)2001
% $Header: /mnt/localhd/cvsdir/MODELLING/NEWDIVA/@ahrbf/private/rbfnet.m,v 1.1.1.1 2006/10/06 18:20:23 brumberg Exp $

% $NoKeywords: $

global in_data out_data;

% Determine the total number of hidden bases
m = varargin{1};    % Input dimension
k = varargin{2};    % Hidden layer description

% If only one value is assigned then each dimension has
% that many units
if (prod(size(k)) == 1),
    k  = k*ones(1,m);
end;
% Otherwise check to ensure that the length of the descriptor has 
% one entry for each input dimension
if (prod(size(k)) ~= m),
    error('rbfnet: no. of bases per dimension not specified');
end;
k = k(:)';

% Create the net structure
net.nin     = m;            % Input dimension
net.hiddist = k;            % Hidden units per dimension
net.nhid    = prod(k);      % Number of hidden nodes
net.nout    = varargin{3};  % Output dimension
lrate       = varargin{4};  % learning rate
irange      = varargin{5};  % input range across dimensions
vers        = varargin{6};  % version of getcenter

% Set the learning rates for the network
net.alpha   = lrate(1);
if length(lrate)>1,
    net.beta    = lrate(2);
else,
    net.beta = 0;
end;

% Initialize values of different net related matrices
net.c = zeros(net.nin,net.nhid);            % distance matrix of input to hidden neurons
net.h = zeros(net.nhid,1);                  % Output of hidden neurons
%net.v = zeros(net.nhid,net.nout);           % Output layer weights
net.v = 0.001.*rand(net.nout,net.nhid);     % Majid's initialization value
net.w = zeros(net.nin,net.nhid,net.nout);   % weights from input to output layer

% Set net range
net.min = min(irange);
net.max = max(irange);
net.range = net.max-net.min; % change to the possibility of diff ranges

% Initialize inputs and outputs
net.out = zeros(net.nout,1);
net.in = zeros(net.nin,1);

% Determine the centers and variances of the networks
net.mu = zeros(net.nin,net.nhid);
net.sg = zeros(net.nin,net.nhid);

% The following piece of code has been subsumed into a more general form
% rf = 1; % range multiplication factor
% for i=1:net.nin,
%     if (k(i)>1),
%         sg(i,1) = (rf*net.range/(k(i)-1))/2;
%     else,
%         sg(i,1) = rf*net.range/2;
%     end;
% end;
% net.sg = repmat(sg,1,net.nhid);

% a and b are temporary storages for net centers
% sg is a temporary storage for variances
% The following loop determines the centers and variances 
% for each input dimension

a = cell(net.nin,1);
for i=1:net.nin,
    % a{i} = getcenter1(rf*net.min,rf*net.max,k(i),i);
    
    % change the function call different ways of initializing the 
    % centers and variances
    switch vers,
    case 1,
        [a{i} sg{i}]= getcenter1(net.min,net.max,k(i),i);
    case 2,
        [a{i} sg{i}]= getcenter2(net.min,net.max,k(i),i);
    case 3,
        [a{i} sg{i}]= getcenter3(net.min,net.max,k(i),i);
    case 4,
        disp('Using k-means to determine center');
    otherwise,
        error('Incorrect version');
    end;
end;

% The following segment of code extends the centers and
% variances for each dimension to a grid across the nin-dimensional
% space
if vers<4,
b = cell(net.nin,1);
c = cell(net.nin,1);
if net.nin == 1,
    b{1} = a{1};
    c{1} = sg{1};
else,
    [b{:}] = ndgrid(a{:});
    [c{:}] = ndgrid(sg{:});
end;

for i=1:net.nin,
    net.mu(i,:) = [b{i}(:)]';
    net.sg(i,:) = [c{i}(:)]';
end;
else,
    [idx,C] = kmeans(in_data',net.nhid);
    net.mu = C';
    for i=1:net.nhid,
%        net.mu(:,i) = mean(in_data(:,idx==i)');
        net.sg(:,i) = var(in_data(:,idx==i)');
    end;
end
% The three different forms of the getcenter function

% FORM 1: Determines the centers and variances based on the range of the
%   the input dimensions and the number of bases for that dimension
function [pos, sg] = getcenter1(netmin,netmax,num_hid,dim_num)
%   netmin: minimum value for the dimension
%   netmax: maximum value for the dimension
%   num_hid: number of hidden neurons for that dimension
%   dim_num: which dimension?

if (num_hid == 1),
    pos = (netmin+netmax)/2;
    sg = (netmax-netmin)/2;
else    
    %pos = linspace(min,max,k); % include the edge in center prediction
    
    % Centers predicted based on number of bins in that dimension
    % Centers are the middle of the bins
    pos = linspace(netmin,netmax,2*num_hid+1);
    pos = pos(2:2:end);
    sg = ((netmax-netmin)/(num_hid-1))/2*ones(num_hid,1);
end;

% FORM 2: Determines the centers and variances based on the range of the
%   the inputs in training data and the number of bases for that dimension
function [pos, sg]= getcenter2(netmin,netmax,num_hid,dim_num)

global in_data out_data

if (num_hid == 1),
    pos = mean(in_data(dim_num,:));
    sg=std(in_data(dim_num,:));
else
    % determine position using bin centers
    pos_i = linspace(netmin,netmax,2*num_hid+1);
    pos_i = pos_i(2:2:end);
    
    for ii=1:num_hid
        kk=find(in_data(dim_num,:)>=netmin+(ii-1)*(netmax-netmin)/num_hid & in_data(dim_num,:)<=netmin+(ii)*(netmax-netmin)/num_hid);
        
        % TODO: alter for the case when no data is present on the bin
        pos(ii)=mean(in_data(dim_num,kk)); 
        
        % 2 seems like an appropriate multiplication factor
        % given uniform sampling
        sg(ii)=2*std(in_data(dim_num,kk));
        kk=[];
    end
end

% Replace the variance of empty bins with an arbitrary large number
idx =find(isnan(sg));
if ~isempty(idx),
    sg(idx) = 10;
end;

% Replace the means of empty bins with centers of bins
idx =find(isnan(pos));
if ~isempty(idx),
    pos(idx) = pos_i(idx);
end;

% FORM 3: Determines the centers and variances based on the range of the
%   the inputs in training data and the number of bases for that dimension
%   using an expectation maximization algorithm

% In development
function [pos, sg]= getcenter3(netmin,netmax,num_hid,dim_num)

global in_data out_data

Nb = num_hid;
[h,x] = hist(in_data(dim_num,:),512);
N = length(h);
f = convn(h,hamming(14),'same');
beta0 = [ones(Nb,1)/Nb,linspace(netmin,netmax,Nb)',4*ones(Nb,1)];
[beta] = gmem(x,f,beta0);
pos = beta(:,2);
%sg = beta(:,3);
if (num_hid == 1),
    sg = (netmax-netmin)/2;
else    
    %pos = linspace(min,max,k); % include the edge in center prediction
    
    % Centers predicted based on number of bins in that dimension
    % Centers are the middle of the bins
    sg = ((netmax-netmin)/(num_hid-1))/2*ones(num_hid,1);
end;
