function varargout=DIVA(varargin)

% DIVA Engine of the DIVA model
%
% DIVA('init' [,SessionName] )                     Initializes the DIVA model
% DIVA('save' [,SessionName] );                    saves state
% DIVA('exit');                                    Exits DIVA model (without saving)
% DIVA(ModuleName,PropertyName [,PropertyValue] )  Reads and writes internal module properties
%
% In general the format [y1,...ym]=DIVA(ModuleName, x1,...xn) is used to interrogate the DIVA
% module 'ModuleName' and for any non-channeled communications with this module.
%
% DIVA('clear');                                   clears all relevant DIVA queues
% DIVA('addmodule',ModuleName,Fcn);                adds a module to the current session
% DIVA('removemodule',ModuleName);                 removes a module from the current session
% DIVA('init' [,SessionName], '-ModuleName1','-ModuleName2',...)
%                                                  initializes DIVA session without
%                                                  loading the specified modules
%
% DIVA('simulate')   performs DIVA simulation.
% DIVA('simulate',n) realizes n seconds of a DIVA simulation.
%
% DIVA(ModuleName, ChannelName, data, delay) sends the vector signal in data to the DIVA
% module 'ModuleName' through the channel 'ChannelName' with a given delay (in seconds).
% If 'data' is a matrix DIVA queues each column of 'data' to be sent consecutively after
% the given delay. Channeled communication with the modules differs from any other
% non-channeled communication in that it is not executed inmediatly, instead it is queued
% to the appropriate time and executed only during a DIVA('simulate') command.
%
% DIVA(ModuleName, ChannelName, [], delay) returns the current data in channel 'ChannelName'
% of the module 'ModuleName' for the time delay(s) specified in the vector 'delay'.
%
% Current modules are (note: module names are case sensitive):
%   Cochlea, SoundChannel, VocalTract, SensoryChannel, Sensory,
%   ForwardModel, AuditoryCortex, SomatosensoryCortex, MotorCortex,
%   AuditoryCortexCaregorical, SoundMap, ModelStatePlot, GUI
%
%   The function DIVA_Template offers a template for creating new module functions.
%    See each specific function (eg. DIVA_cochlea) for channel
%    definitions and functionality.
%
% Current DIVA properties are: (* requires re-initialization)
%    Modules:     Contains a structure with the matlab functions associated with each
%                 DIVA module.
% *  StorageTime: Maximum time (in seconds) stored in the DIVA engine queues.
% *  TimeStep:    Time-step (in seconds) for DIVA simulations
%    PlotStep:    Time-step (in milliseconds) for DIVA state variable plot updates
%    BlockMode:   0: The DIVA engine provides the modules with one vector of data at a time;
%                 1: The DIVA engine provides the modules with matrices containing all the data
%                    simulatenously
%                 (Note: when 'BlockMode' equals to 1, the real-time error correction will not
%                        be implemented)
%
% Examples:
% To initiate a DIVA session and begin a simulation
% DIVA('init'); % initialize DIVA
% DIVA('SoundChannel','sound',s); % send sound vector to the SoundChannel
% DIVA('AuditoryCortexCategorical','label',label); % identify incoming sound
% DIVA('simulate'); % Acquire new sound and label
%
% DIVA('AuditoryCortexCategorical','target',label); % Select auditory target to train
% DIVA('simulate'); % Simulate DIVA learning procedure
%
% To start DIVA without specific modules:
% DIVA('init',Sessionname,'-GUI'); % To start DIVA without the GUI
%
% 25-03-2007: Removed Microphone module, not compatible with formant
% version

  global DIVA_data
  if nargin<1 || (ischar(varargin{1}) && length(varargin{1})>1 && varargin{1}(1)=='-'),
    % Test for empty or "removemodule (-)" arguments
    [varargin{1:nargin+2}] = deal('init','default',varargin{:});
    nnargin=2+nargin;
  else,
    nnargin=nargin;
  end

  switch(lower(varargin{1})),
   case 'init', % Initialization
    if nnargin>1 && varargin{2}(1)~='-',
      % init with specified session
      sessionname=lower(varargin{2});
    else,
      % assume default session
      sessionname='default';
    end
    % add paths for auxillary functions (fcn/, i.e. GUI functions) and data
    % (i.e. cortical surface, Peterson and Barney db)
    addpath(genpath([fileparts(which(mfilename)),filesep,'fcn']));
    addpath(genpath([fileparts(which(mfilename)),filesep,'data']));

    SaveFolder=[fileparts(which(mfilename)),filesep,'Sessiondata',filesep,...
                'Session_',sessionname,filesep];

    if isempty(dir([SaveFolder,mfilename,'.mat'])),
      % Read-write fields
      DIVA_data.params=struct(...
          'SessionFolder','Sessiondata',...
          'SessionTargetFolder','Sessiontargets',...
          'TimeStep',.001,...   % Timestep (in seconds) for simulations
          'PlotStep',25,...     % Timestep for output plot update
          'StorageTime',10,...  % Maximum time (in seconds) stored by the DIVA engine
          'BlockMode',1,...     % See above
          'Now',1,...
          'status','on',...
          'Modules',struct(...                          % Module name, Module function
              'Cochlea',@DIVA_Cochlea,...               % avoid problems with dependencies in the
              'SoundChannel',@DIVA_SoundChannel,...     % initialization; e.g. AuditoryCortex
              'VocalTract',@DIVA_VocalTract,...         % module initialization interrogates
              'SensoryChannel',@DIVA_SensoryChannel,... % Cochlea module properties => cochlea
              'Sensory',@DIVA_Sensory,...               % needs to initialize first)
              'ForwardModel',@DIVA_ForwardModel,...
              'AuditoryCortex',@DIVA_AuditoryCortex,...
              'SomatosensoryCortex',@DIVA_SomatosensoryCortex,...
              'AuditoryCortexCategorical',@DIVA_AuditoryCortexCategorical,...
              'MotorCortex',@DIVA_MotorCortex,...
              'SoundMap',@DIVA_SoundMap,...
              'ModelStatePlot',@DIVA_ModelStatePlot,...
              'GUI',@DIVA_GUI));
    else
      data=load([SaveFolder,mfilename,'.mat'],'-mat');
      DIVA_data=data.params;
    end
    % Parameters initialized for every session
    DIVA_data.params.Nstep=ceil(DIVA_data.params.StorageTime/DIVA_data.params.TimeStep);
    DIVA_data.params.timer=timer('TimerFcn',{@DIVA,'step'},...
                                 'Period',DIVA_data.params.TimeStep,...
                                 'ExecutionMode','fixedRate',...
                                 'BusyMode','queue');

    % QueueStatus indicates whether a queue item has been previously seen
    % -> Queues are circular
    DIVA_data.params.QueueStatus=zeros([1,DIVA_data.params.Nstep]);
    % busy indicates if any module is currently computing
    % -> Can change such that there is heirarch of busy, for each layer of model
    DIVA_data.params.busy=[0,0];
    DIVA_data.params.SessionName=sessionname;

    if strcmp(DIVA_data.params.SessionName,'empty'),
      DIVA_data.params.Modules=struct('GUI',@DIVA_GUI);
    end; % option to initialize an empty DIVA session

    for n1=1:nnargin, % option to remove particular modules during initialization
      if varargin{n1}(1)=='-',
        DIVA_data.params.Modules=rmfield(DIVA_data.params.Modules,varargin{n1}(2:end));
      elseif varargin{n1}(1)=='+',
        DIVA_data.params.Modules=setfield(DIVA_data.params.Modules,...
                                          varargin{n1}(2:end),...
                                          ['DIVA_',varargin{n1}(2:end)]);
      end
    end

    % Modules initialization
    warning('off','MATLAB:warn_r14_stucture_assignment');
    DIVA_data.Modules=DIVA_data.params.Modules;
    % For each module, call its 'init' procedure
    for fcn=fieldnames(DIVA_data.params.Modules)',
      out=feval(DIVA_data.params.Modules.(fcn{1}),'init',sessionname);
      % Construct list of channels based on initial "out" parameters from modules
      DIVA_data.Modules.(fcn{1}).Channels=struct;
      for channel=1:length(out),
        % Initialize channel queues
        DIVA_data.Modules.(fcn{1}).Channels.(out{channel})=zeros(0,DIVA_data.params.Nstep);
      end
      % Initialize channel queue read indicators
      DIVA_data.Modules.(fcn{1}).ChannelsTouch=zeros([length(out),DIVA_data.params.Nstep]);
    end
    %*************************** END INIT ******************************%
    
   case 'status',
    if nnargin<2,
      if isempty(DIVA_data),
        varargout{1}='off';
      else,
        varargout{1}=DIVA_data.params.status;
      end
    else,
      DIVA_data.params.status=varargin{2};
    end
    %**************************** END STATUS **************************%
    
   case 'clear', % Clear DIVA queues
    % Reset all DIVA params to initial values
    DIVA_data.params.busy=[0,0];
    DIVA_data.params.Now=1;
    fcns=fieldnames(DIVA_data.params.Modules)';
    for fcn=fcns,
      channels=fieldnames(DIVA_data.Modules.(fcn{1}).Channels)';
      for channel=channels,
        DIVA_data.Modules.(fcn{1}).Channels.(channel{1})=zeros(0,DIVA_data.params.Nstep);
      end
      DIVA_data.Modules.(fcn{1}).ChannelsTouch=...
          zeros([length(channels),DIVA_data.params.Nstep]);
    end
    DIVA_data.params.QueueStatus=zeros([1,DIVA_data.params.Nstep]);
    %**************************** END CLEAR ****************************%
    
   case 'reset',
    % Reset all learned targets (somatosensory) and feedforward command
        % clear model weight parameters, need to clear state values too!
    if(nargin==2)
      DIVA('AuditoryCortexCategorical','target',varargin{2});
    end
    matchstart = DIVA('AuditoryCortexCategorical','ntimepoints')*...
        DIVA('AuditoryCortexCategorical','match')+1;
    matchend=matchstart+DIVA('AuditoryCortexCategorical','ntimepoints')-1;

    % Reset Motor Weights
    motorweight=DIVA('MotorCortex','WeightsFromTargets');
    motorweight(:,matchstart:matchend)=...
        repmat(DIVA('MotorCortex','default_motorSound'),1,...
               DIVA('AuditoryCortexCategorical','ntimepoints'));
    DIVA('MotorCortex','WeightsFromTargets',motorweight);

    % Reset Somatosensory Weights
    somatoweight=DIVA('SomatosensoryCortex','WeightsFromTargets');
    somatoweight(:,matchstart:matchend)=...
        repmat(DIVA('SomatosensoryCortex','default_somatosensory'),1,...
               DIVA('AuditoryCortexCategorical','ntimepoints'));
    DIVA('SomatosensoryCortex','WeightsFromTargets',somatoweight);
    
    % Reset stored training trials
    tmp=DIVA('AuditoryCortexCategorical','AuditoryTargets');
    tmp(DIVA('AuditoryCortexCategorical','match')+1).ntrained=0;
    DIVA('AuditoryCortexCategorical','AuditoryTargets',tmp);

    
    %**************************** END RESET ****************************%
    
   case 'plot', % Plot all model traces in external figure
    if(nargin>1),
      idx=varargin{2};
    else
      idx=DIVA('TimeStep')*(-1000:0);
    end
    
    match=DIVA('AuditoryCortexCategorical','match');
    au_info=DIVA('AuditoryCortexCategorical','AuditoryTargets');
    name=au_info(match+1).label;
    ntimepoints=DIVA('AuditoryCortexCategorical','ntimepoints');
    aut=DIVA('AuditoryCortex','WeightsFromTargets');
    aut=aut(:,match*ntimepoints+1:match*ntimepoints+ntimepoints);
    aut_idx=DIVA('AuditoryCortex','target',[],idx);
    aut_idx=max(round(aut_idx)-(match*ntimepoints),0);
    
    tmp=ones(size(aut,1),length(aut_idx));
    tmp(:,find(aut_idx==0))=NaN;
    tmp(:,find(aut_idx~=0))=aut(:,aut_idx(find(aut_idx~=0)));
    aut=tmp;
    au=DIVA('AuditoryCortex','sound',[],idx);
    m=DIVA('VocalTract','motor',[],idx);
    s=DIVA('SomatosensoryCortex','somatosensory',[],idx);
    dau=DIVA('MotorCortex','error_sound',[],idx);
    ds=DIVA('MotorCortex','error_somatosensory',[],idx);
    
    if(nargout),
      tmp.sound=au;
      tmp.motor=m;
      tmp.somatosensory=s;
      tmp.error_sound=dau;
      tmp.error_somatosensory=ds;
      varargout{1}=tmp;
    end
    figure,
    subplot(5,1,1),plot(au','r'),hold on, plot(aut','k');title('Auditory')
    subplot(5,1,2),plot(m');title('Motor')
    subplot(5,1,3),plot(s');title('Somatosensory')
    subplot(5,1,4),plot(dau');title('Auditory Error')
    subplot(5,1,5),plot(ds');title('Somatosensory Error')
    
    %**************************** END PLOT *****************************%
   
   case 'addmodule', % Add module to running DIVA instance
    DIVA_data.params.Modules.(varargin{2})=varargin{3};
    out=feval(DIVA_data.params.Modules.(varargin{2}),'init',DIVA_data.params.SessionName);
    DIVA_data.Modules.(varargin{2}).Channels=struct;
    for channel=1:length(out),
      DIVA_data.Modules.(varargin{2}).Channels.(out{channel})=...
          zeros(0,DIVA_data.params.Nstep);
    end
    DIVA_data.Modules.(varargin{2}).ChannelsTouch=...
        zeros([length(out),DIVA_data.params.Nstep]);
    %************************* END ADDMODULE ****************************%

   case 'removemodule', % Remove module from running DIVA instance
    if ~isfield(DIVA_data.params.Modules,varargin{2}),
      warning([mfilename, ' wrong module name ',varargin{2}]);
    else,
      DIVA(varargin{2},'exit');
      DIVA_data.params.Modules=rmfield(DIVA_data.params.Modules,varargin{2});
    end
    %*********************** END REMOVEMODULE ***************************$

   case 'save', % Save DIVA state
    if nnargin>1,
      sessionname=lower(varargin{2});
    else,
      sessionname=DIVA_data.params.SessionName;
    end
    % Create session Directory
    Folder1=[fileparts(which(mfilename)),filesep];
    Folder2=[DIVA_data.params.SessionFolder,filesep];
    Folder3=['Session_',sessionname,filesep];
    if ~exist([Folder1,Folder2],'dir'),
      mkdir(Folder1,Folder2);
    end
    if ~exist([Folder1,Folder2,Folder3],'dir'),
      mkdir([Folder1,Folder2],Folder3);
      disp(['Creating Directory : ',Folder1,Folder2,Folder3]);
    end
    DIVA_data.params.SessionName=sessionname;
    % Save all modules -> See "*" below
    DIVA('*','save',sessionname);
    params=DIVA_data;
    save([Folder1,Folder2,Folder3,mfilename,'.mat'],'params');
    %*************************** END SAVE **********************%
    
   case 'exit', % Exit DIVA instance
    DIVA('status','off');
    DIVA('*','exit');
    delete(DIVA_data.params.timer);
    clear DIVA_data;
    %*********************** END EXIT **************************%
    
   case 'disp', % Output all DIVA and Module parameters to the screen
    disp(['--------- ','DIVA',' ---------']);
    disp(DIVA_data.params);
    fcn=fieldnames(DIVA_data.params.Modules);
    for n1=1:length(fcn),
      disp(['--------- ',fcn{n1},' ---------']);
      feval(DIVA_data.params.Modules.(fcn{n1}),'disp');
    end    
    %********************* END DISP ****************************%
    
   case '*', % DO ALL -> perform action specified in varargin{2:end} to all Modules
    fcn=fieldnames(DIVA_data.params.Modules);
    for n1=1:length(fcn),
      varargout{n1}=feval(DIVA_data.params.Modules.(fcn{n1}),varargin{2:end});
    end
    %********************* END DOALL *****************************%
    
   case 'simulate', % Begin DIVA Simulation
    
    if nnargin<2,
      % Step through DIVA Queues and perform module actions through 
      % channel communication, until there is no new information
      % in the queue
      while any(DIVA_data.params.QueueStatus),
        DIVA('step');
      end
    else,
      if strcmpi(varargin{2},'realtime'),
        DIVA('start');
      else,
        % Step through Queues for varargin{2} seconds
        for n1=1:ceil(varargin{2}/DIVA_data.params.TimeStep),
          DIVA('step');
        end
      end
    end
    %********************* END SIMULATE ***************************%
    
   case 'start'
    start(DIVA_data.params.timer);

   case 'stop'
    stop(DIVA_data.params.timer);

   case 'step',
    if ~any(DIVA_data.params.busy),
      DIVA_data.params.busy(1) = 1;  % Set DIVA busy flag
      Now=DIVA_data.params.Now;      % Now is pointer to current queue location
      NowAndFuture = ...             % NowAndFuture is pointer to next queue location
          1 + mod(DIVA_data.params.Now - 1 +...
                  (0:DIVA_data.params.Nstep -1),...
                  DIVA_data.params.Nstep);
      for fcn=fieldnames(DIVA_data.params.Modules)',
        % Identify channels per module
        channels = fieldnames(DIVA_data.Modules.(fcn{1}).Channels);
        channelsTouch = find(DIVA_data.Modules.(fcn{1}).ChannelsTouch(:,Now));
        
        if length(channelsTouch) > 0,
          if DIVA_data.params.BlockMode,
            % Find index of next block of channel information ( > 1)
            [nill,idx] = min(DIVA_data.Modules.(fcn{1}).ChannelsTouch(:,NowAndFuture),[],2);
            idx = 1:min(idx(idx > 1)) - 1;
          else,
            % Increment index for channel information by one
            idx = 1;
          end
          options = [];
          % Determine channel information to send to Modules
          for channel=1:length(channelsTouch),            
            options{2 * channel - 1} = channels{channelsTouch(channel)};
            options{2 * channel} = DIVA_data.Modules.(fcn{1}).Channels.(channels{channelsTouch(channel)})(:,NowAndFuture(idx));
          end
          % Send channel information
          feval(DIVA_data.params.Modules.(fcn{1}),options{:});
          DIVA_data.Modules.(fcn{1}).ChannelsTouch(:,NowAndFuture(idx)) = 0;
        end
      end
      % Indicate current queue element as "read" and move queue pointer to next index
      DIVA_data.params.QueueStatus(Now) = 0;
      DIVA_data.params.Now = 1 + mod(DIVA_data.params.Now,DIVA_data.params.Nstep);
      DIVA_data.params.busy(1) = 0;
    end
    %********************** END STEP ************************************%
    
   otherwise, % Manages module channels and properties
    if isfield(DIVA_data.params,varargin{1}), % Reads/Writes properties
      if nnargin < 2,
        if nargout > 0, % output properties
          [varargout{1:nargout}] = DIVA_data.params.(varargin{1});
        end
      else, % input properties
        DIVA_data.params.(varargin{1}) = varargin{2};
      end
    else, % Get/Set queue information
      DIVA_data.params.busy(2) = 1;
      indexargin = 1;
      while indexargin < nnargin,
        if isfield(DIVA_data.params.Modules,varargin{indexargin}),
          channel=strmatch(varargin{indexargin + 1},...
                           fieldnames(DIVA_data.Modules.(varargin{indexargin}).Channels),...
                           'exact');

          if length(channel) > 0 && ~nargout, 
          % Writes Channel
            if nnargin < indexargin + 3,
              varargin{indexargin + 3} = 0;
            end
            Now = 1 + mod(DIVA_data.params.Now - 1 +...
                          max(1,round(varargin{indexargin+3}/DIVA_data.params.TimeStep))+...
                          (0:size(varargin{indexargin+2},2)-1),DIVA_data.params.Nstep);
            if length(Now)>DIVA_data.params.Nstep, ...
                  warning(['DIVA: Data too large for queuing. Increase StorageTime in ',...
                         mfilename]);
            end            
            DIVA_data.Modules.(varargin{indexargin}).Channels.(varargin{indexargin+1})(1:size(varargin{indexargin+2},1),Now) = varargin{indexargin+2};
            DIVA_data.Modules.(varargin{indexargin}).ChannelsTouch(channel,Now) = 1;
            DIVA_data.params.QueueStatus(Now) = 1;
            indexargin=indexargin+4;
            
          elseif length(channel)>0 && isempty(varargin{indexargin+2}), 
            % Reads Channel,
            Now=1+mod(DIVA_data.params.Now-1+...
                      round(varargin{indexargin+3}/DIVA_data.params.TimeStep),...
                      DIVA_data.params.Nstep);
            if any(varargin{indexargin+3}<-DIVA_data.params.StorageTime),
              warning(['DIVA: Delay too long for queue. Increase StorageTime in ',mfilename]);
            end
            varargout{1} = ...
                DIVA_data.Modules.(varargin{indexargin}).Channels.(varargin{indexargin+1})(:,Now);
            indexargin=indexargin+4;
            
          elseif ~isempty(DIVA_data.params.Modules.(varargin{indexargin})),
            % Non-channeled communication
            if ~nargout,
              feval(DIVA_data.params.Modules.(varargin{indexargin}),...
                    varargin{indexargin+1:end});
            else,
              [varargout{1:nargout}]=...
                  feval(DIVA_data.params.Modules.(varargin{indexargin}),...
                        varargin{indexargin+1:end});
            end
            indexargin=nnargin;
          end
        else,
          warning(['warning: ',mfilename,' non-existing module name ',...
                   varargin{indexargin}]);
          indexargin=nnargin;
        end
      end
      DIVA_data.params.busy(2)=0;
    end
  end

