function out=DIVA_AuditoryCortex(varargin);
% DIVA_AuditoryCortex Auditory cortex model
%
% This model computes the cortical representation of a
% a sound and its difference with the target sound.
%
% DIVA_AuditoryCortex('init' [,SessionName]);          Initializes the module
% DIVA_AuditoryCortex('save' [,SessionName] );         Saves state
% DIVA_AuditoryCortex('exit');                         Exits the module (without saving)
% DIVA_AuditoryCortex(PropertyName [,PropertyValue] )  Reads and writes internal model
%                                                      properties
%
% DIVA_AuditoryCortex('sound',s1);
% The model computes the cortical sound representation of the sound s1
% and passes it to the SoundMap area for categorical perception.
%
% DIVA_AuditoryCortex('target',s2,'sound',s1);
% The model computes the difference between the target signal s2
% and the sound signal s1 and sends it to motor cortex (DIVA_MotorCortex).
%
% The signal s1 is a cochlear representation of a sound (Formant
% vector, see DIVA_Cochlea) and the signal s2 is a categorical
% representation (Formant vector with min and max values, see
% DIVA_SoundMap) of the sound. The model stores the weights, 
% WeightsFromTargets used to compare incoming auditory input with
% stored weights
%
% Current DIVA_AuditoryCortex properties are: (* requires re-initialization)
%  *  delayToSoundMap        : delay (in seconds) for signals to SoundMap
%  *  delayToMotorCortex     : delay (in seconds) for signals to SoundMap
%     WeightsFromTargets     : weights for target signals
%


out=[];
global DIVA_AuditoryCortex_data

%%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%
%%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

for indexargin=1:2:nargin,
  switch(varargin{indexargin}),
   case 'init',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1 | isempty(varargin{indexargin+1}),
      initfile='';
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end

    if isempty(initfile) || isempty(dir(initfile)),
      disp([mfilename, ' : Defining new session...']);
      DIVA_AuditoryCortex_data.params=struct(...
          'delayToSoundMap',.005,...
          'delayToMotorCortex',.005,...
          'WeightsFromTargets',[],... % stored as matrix [[lower;upper]:ntimepoints]
          'error_sound',[],...        % auditory error
          'nFormants',3);             % Dimension of auditory representation

    else,
      data=load(initfile,'-mat');
      DIVA_AuditoryCortex_data.params=data.params;
    end
    DIVA_AuditoryCortex_data.params.TimeStep=DIVA('TimeStep');
    DIVA_AuditoryCortex_data.params.current.target=[];
    DIVA_AuditoryCortex_data.params.current.sound=[];
    % Channels
    out={'target','sound'};

   case 'save',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1,
      initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end
    params=DIVA_AuditoryCortex_data.params;
    save(initfile,'params');

   case 'exit',
    clear DIVA_AuditoryCortex_data;

   case 'disp',
    disp(DIVA_AuditoryCortex_data.params);
    out=fieldnames(DIVA_AuditoryCortex_data.params);

   case 'WeightsFromTargets', % overloaded operator (plot to GUI)
    if indexargin==nargin,
      out=DIVA_AuditoryCortex_data.params.(varargin{indexargin});
    else,
      DIVA_AuditoryCortex_data.params.(varargin{indexargin})=varargin{indexargin+1};
      DIVA('GUI','AC_WeightsFromTargets','target');
    end

    %%%%%%%%%%%%%%%% ACTION %%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

   case 'target'
    % current sound target projection
    DIVA_AuditoryCortex_data.params.current.target = varargin{indexargin+1};
    if ~nargout,
      DIVA('ModelStatePlot','AuditoryCortex','target');
    end

   case 'sound'
    currentsound = varargin{indexargin+1};

    if isempty(DIVA_AuditoryCortex_data.params.current.target),
      % if there is no current target, pass the sound representation to the SoundMap area
      if ~nargout,
        oldcurrentsound = DIVA_AuditoryCortex_data.params.current.sound;
      else,
        oldcurrentsound = [];
      end

      out = currentsound;

      if ~nargout,
        DIVA_AuditoryCortex_data.params.current.sound = ...
            [DIVA_AuditoryCortex_data.params.current.sound,currentsound];

        % Send to Categorical Auditory Cortex
        DIVA('AuditoryCortexCategorical','sound',out,...
             DIVA_AuditoryCortex_data.params.delayToSoundMap);
      end

    else,
      % if there is a current target, compute the error and send it
      % to the PerceptionProduction area
      
      % indices from SoundMap are real, not integer (see
      % AuditoryCortexCategorical for more), so compute appropriate
      % integer indicies
      idx = union(floor(DIVA_AuditoryCortex_data.params.current.target),...
                  ceil(DIVA_AuditoryCortex_data.params.current.target));

      if length(idx)>1,
        % Non-stationary target (i.e. CV*)
        DIVA_AuditoryCortex_data.params.current.targetsound = ...
            fastinterp1(idx,DIVA_AuditoryCortex_data.params.WeightsFromTargets(:,idx),...
                        DIVA_AuditoryCortex_data.params.current.target);
      else,
        % Steady-state target (i.e. V)
        DIVA_AuditoryCortex_data.params.current.targetsound = ...
            DIVA_AuditoryCortex_data.params.WeightsFromTargets(:,idx*ones(1,size(DIVA_AuditoryCortex_data.params.current.target,2)));
      end


      % Compute Error between current produced auditory signal and
      % stored targets
      %
      % Auditory error assumed 0 when within stored target regions
      % (upper and lower below)

      upper = DIVA_AuditoryCortex_data.params.current.targetsound(4:6,:);
      lower = DIVA_AuditoryCortex_data.params.current.targetsound(1:3,:);

      if ~nargout,
        oldcurrentsound = DIVA_AuditoryCortex_data.params.current.sound;
     else,
       oldcurrentsound = [];
     end

     % Calculate mean auditory target in case developers want to
     % add additional emphasis on moving to the center of the
     % target region, currently not used!
     for n=1:DIVA_AuditoryCortex_data.params.nFormants,
       meansound(n,:) = mean([upper(n,:);lower(n,:)],1);
     end     
     
     % Determine error between auditory signal and target regions
     out= min(0,upper-currentsound)+max(0,lower-currentsound);
     
     % Determine error between auditory signal and target regions with force towards center
     %  out = (meansound-currentsound).*(currentsound>upper) + ...
     %        (meansound-currentsound).*(currentsound<lower) + ...
     %        (meansound-currentsound)*0.1*(currentsound<=upper & currentsound>=lower);
     
     
     % [fmt,errorIdx]=find(any(currentsound<(upper+0.01*(upper-lower)) & ...
     %                         currentsound>(lower-0.01*(upper-lower))));
     
     % Add inverse-gaussian weighted error for areas within the target region
     %if(any(currentsound<(upper+0.05*(upper-lower)) & currentsound>(lower-0.05*(upper-lower))))
     % if(~isempty(errorIdx))       
     %   errorWeightMax=normpdf(0,0,[upper(:,errorIdx)-lower(:,errorIdx)]/3); % for 3*std 97% variance
     %   errorWeight=...
     %       (1-normpdf((meansound(:,errorIdx)-currentsound(:,errorIdx)),0,...
     %                  [upper(:,errorIdx)-lower(:,errorIdx)]/3)/errorWeightMax) *...
     %       (currentsound(:,errorIdx)-meansound(:,errorIdx));
     %   out(:,errorIdx)= out(:,errorIdx)+errorWeight;
  
% end
     if ~nargout, % Send auditory error to MotorCortex
       DIVA_AuditoryCortex_data.params.current.sound = ...
           [DIVA_AuditoryCortex_data.params.current.sound,currentsound];
       DIVA_AuditoryCortex_data.params.current.error=out;
       DIVA('MotorCortex','error_sound',...
            DIVA_AuditoryCortex_data.params.current.error,...
            DIVA_AuditoryCortex_data.params.delayToMotorCortex);
     end
     DIVA_AuditoryCortex_data.params.current.target=[];
    end

    if any(any(isnan(varargin{indexargin+1}))),
      DIVA_AuditoryCortex_data.params.current.sound=[];
    end

    if ~nargout, % Attempt to plot relevant information in on the
                 % GUI and ModelStatePlot
      DIVA('ModelStatePlot','AuditoryCortex','sound');
      DIVA('GUI','AC_WeightsFromTargets','update');
      DIVA('GUI','other_weights','auditory');
    end

   otherwise, % Handle AuditoryCortex Parameters (read/write)
    if isfield(DIVA_AuditoryCortex_data.params,varargin{indexargin}),
      if indexargin==nargin,
        out=DIVA_AuditoryCortex_data.params.(varargin{indexargin});
      else,
        DIVA_AuditoryCortex_data.params.(varargin{indexargin})=varargin{indexargin+1};
      end
    else,
      warning('DIVA_AuditoryCortex: wrong argument');
    end

  end
end
