function out=DIVA_AuditoryCortexCategorical(varargin)
% DIVA_AuditoryCortexCategorical Categorical perception model
%
% This model initiates the production of phonemic segments,
% sending an index to the desired phonemic target to MotorCortex,
% and the expected auditory and somatosensory consequences
% of the phonemic target to AuditoryCortex and SomatosensoryCortex
% respectively via the SoundMap.  There is a one-to-one relationship
% between AuditoryCortexCategorical 'target's and SoundMap
%
% This model also recognizes sound segments (from AuditoryCortex)
% by comparing them to stored phonemic representations (if not
% match is found the model creates a new phonemic representation
% for the present sound). Phonemic segments are defined by
% blocks of sound surrounded by silence.
%
% DIVA_AuditoryCortexCategorical('init' [,SessionName]);          Initializes the module
% DIVA_AuditoryCortexCategorical('save' [,SessionName] );         Saves state
% DIVA_AuditoryCortexCategorical('exit');                         Exits the module (without saving)
% DIVA_AuditoryCortexCategorical(PropertyName [,PropertyValue] )  Reads and writes internal model
%                                                                 properties
%
% DIVA_AuditoryCortexCategorical('label',PhonemeName);
% Informs the AuditoryCortexCategorical model that the next sound segment
% should be identified as the phonemic segment 'PhonemeName'.
%
% DIVA_AuditoryCortexCategorical('sound',s);
% Sends the sound signal s (AuditoryCortex representation) to the
% AuditoryCortexCategorical model. This is sequentially stored as the current sound
% segment until a period of silence (all zeros) is found.
% Then, if a pre-existing phonemic label has been associated
% with this segment its phonemic representation is updated, otherwise
% the model compares the incoming sound segment with the stored
% phonemic representations and displays the most likely match.
%
% DIVA_AuditoryCortexCategorical('target',PhonemeName);
% Initiates production of phonemic target 'PhonemeName'. The
% model sends sequentially indexes to a temporal representation
% of the phonemic target to the SoundMap area.
%
% Current DIVA_AuditoryCortexCategorical properties are: (* requires re-initialization)
%     AuditoryTargets         : structure containing the learned auditory targets
%                               (read-only)
%     SilenceThresholdEnergy  : Threshold for sound segments limits
%                               (Energy must be larger than 'SilenceThresholdEnergy')
%                               [Not necessary since formants always > 0]
%     SilenceThresholdTime    : for at least 'SilenceThresholdTime' seconds)
%                               Note: Both SilenceThresholdEnergy/Time are important for parsing
%                                     continuous waveform input (i.e. microphone) and utilized in
%                                     the spectral representation version of the model, not in
%                                     (formant) version
%     delayToSoundMap         : delay (in seconds) for signals to SoundMap
%     ntimeponts              : number of interpolated time-points in the stored representations
%                               Auditory Targets are time-normalized by representing areas of
%                               higher auditory trajectory velocity than more static ones.  The
%                               number of normalized time-points is specified by ntimepoints.
%                               The target velocity is stored within the target specification for
%                               use in reconstruction of the original time scale as well as
%                               velocity scaling the stored representation
%

% Note: This model updates the target representation of the
%       AuditoryCortex area to match the AuditoryTargets
%       stored here (non-modular design, to be changed later...)

% 2006-10-18: (JSB) can produce learned sounds in sequence **no
% coarticulation.  See 'target' functionality

% Dependencies: equalstep.m

% TODO % Allow user variation of target regions, global and local

out=[];
global DIVA_AuditoryCortexCategorical_data

for indexargin=1:2:nargin,
  switch(varargin{indexargin}),
   case 'init',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1 || isempty(varargin{indexargin+1}),
      initfile='';
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end
    if isempty(initfile) || isempty(dir(initfile)),
      disp([mfilename, ' : Defining new session...']);
      % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%
      DIVA_AuditoryCortexCategorical_data.params=struct(...
          'ntimepoints',64,...    % number of interpolated time-points in the stored representations
          'delayToSoundMap',.005,...
          'SilenceThresholdEnergy',.05,... % Threshold for sound segments (Energy larger than
          'SilenceThresholdTime',.050,...  % 'SilenceThresholdEnergy' for at least 'SilenceThresholdTime' seconds)
          'match',-1,...                   % Currently selected auditory target
          'AuditoryTargets',...       % Auditory Target Specification
          repmat(struct(...
              'mean',[],...           % Mean signal (formants)
              'var',[],...            % Signal Variance, used for target regions
              'velocity',[],...       % Signal velocity, used for normalization
              'label',[],...          % Phoneme/syllable/sound identification
              'nsamples',[],...	      % Number of samples, used to adjust variance
              'ntrained',[]),[1,0])); % Number of training trials
      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    else,
      data=load(initfile,'-mat');
      DIVA_AuditoryCortexCategorical_data.params=data.params;
    end
    % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    DIVA_AuditoryCortexCategorical_data.params.TimeStep = DIVA('TimeStep');
    % nbinsenery used in Spectral implementation, removed by JSB for formant version
    % DIVA_AuditoryCortexCategorical_data.params.nbinsenergy = ...
    %     DIVA('AuditoryCortex','nbinsenergy');
    DIVA_AuditoryCortexCategorical_data.params.current=struct(...
        'SoundSegment',[],...
        'PhonemeLabel',[]);
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % LIST OF MODULE CHANNELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    out={'sound'};
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %********************* END INIT ****************************%

   case 'save',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1,
      initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end
    params=DIVA_AuditoryCortexCategorical_data.params;
    save(initfile,'params');
    %********************* END SAVE ****************************%

   case 'exit',
    clear DIVA_AuditoryCortexCategorical_data;
    % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    %********************* END EXIT ****************************%
   case 'disp', % Display Module Parameters
    disp(DIVA_AuditoryCortexCategorical_data.params);
    out=fieldnames(DIVA_AuditoryCortexCategorical_data.params);
    %********************* END DISP ****************************%

    %-----------------------------------------------------------%

   case 'sound', % Updates current sound segment info
    idx = find(~any(isnan(varargin{indexargin+1})));
    DIVA_AuditoryCortexCategorical_data.params.current.SoundSegment(:,end+(1:length(idx))) = ...
        varargin{indexargin+1}(:,idx);

    % Sound segment complete: recognition and updating of phonemic targets
    if isempty(find(idx==size(varargin{indexargin+1},2))) && ...
          ~isempty(DIVA_AuditoryCortexCategorical_data.params.current.SoundSegment),
      disp('DIVA_AuditoryCortexCategorical: Received sound segment');


      % Time normalize signal, SoundSegment = normalized signal, vel = transfer function
      e1 = DIVA_AuditoryCortexCategorical_data.params.current.SoundSegment;
      e1 = mean(e1./repmat(max(e1,[],2),1,size(e1,2)),1);
      n = ceil(DIVA_AuditoryCortexCategorical_data.params.SilenceThresholdTime/...
               DIVA_AuditoryCortexCategorical_data.params.TimeStep);

      % Find valid indicies of input signal (i.e. area between silences, assumed to be entire
      % signal in formant version)

      if(length(e1)==1)
        e2=1;
      else
        e2=find(min(sample_window(e1',n,n-1),[],1));
      end
      if ~length(e2),
        if(length(e1)>0),
          disp('DIVA_AuditoryCortexCategorical: Sound segment contains only silence');
        end
      else,
        if length(e2)>1, %dynamic target
          lims = max(1,min(length(e1),[e2(1)-n,e2(end)+2*n-1]));
          % normalization (SoundSegment is interpolated to a fixed number of timepoints)
          % by assigning more timepoints to parts of the SoundSegment with higher velocity
          [SoundSegment,time] = ...
              equalstep(DIVA_AuditoryCortexCategorical_data.params.current.SoundSegment(:,lims(1):lims(2)),...
                        DIVA_AuditoryCortexCategorical_data.params.ntimepoints,'fixedlength');
          time = time*DIVA_AuditoryCortexCategorical_data.params.TimeStep;
          vel  = 1./diff(time);
        else, % static target
          SoundSegment = ...
              DIVA_AuditoryCortexCategorical_data.params.current.SoundSegment(:,ones(1,DIVA_AuditoryCortexCategorical_data.params.ntimepoints));
          vel = 0;
        end

        % JSB: Recognition is OK, simple MSE calculation alright due to simple, formant basis
        % recognition
        matchlevels = ...
            nan + zeros(1,length(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets));

        % JSB: For now, no actual recognition of input signal, just assume user is correct
        %      when inputting 'label'
        %      Need to implement better recognition (formant based) for use in calculating
        %      appropriate signal variances (and means) rather than hardcode [400,2500,10000] Hz
        %      variance

        %  for n1=1:length(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets),
        %    % Determine MSE between current sound and stored targets
        %    matchlevels(n1) = ...
        %        mean(mean(...
        %            (DIVA_AuditoryCortexCategorical_data.params.current.SoundSegment-...
        %             DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(n1).mean).^2,1),2)
        %  end
        %  [matchlevel,winner]=min(matchlevels)
        %  if ~isempty(winner),
        %    disp(['DIVA_AuditoryCortexCategorical: Best match to sound segment ',...
        %          DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(winner).label]);
        %  end

        % JSB: Store/update current phoneme representation

        if ~isempty(DIVA_AuditoryCortexCategorical_data.params.current.PhonemeLabel),
          % update phoneme representation
          match = strmatch(DIVA_AuditoryCortexCategorical_data.params.current.PhonemeLabel,...
                           {DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(:).label},...
                           'exact');

          %% JSB: New Phonemic Target -- OK
          if isempty(match), % new phonemic target
            match = length(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets)+1;
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean = SoundSegment;
            % Hardcode variance for now
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).var = ...
                repmat([400;2500;10000],1,size(SoundSegment,2));
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).velocity=vel;
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).label = ...
                DIVA_AuditoryCortexCategorical_data.params.current.PhonemeLabel;
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples = 1;
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).ntrained = 0;

            % updates AuditoryCortex targets too
            temp = DIVA('AuditoryCortex','WeightsFromTargets');
            temp(1:2*size(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean,1),...
                 DIVA_AuditoryCortexCategorical_data.params.ntimepoints*(match-1)+...
                 (1:DIVA_AuditoryCortexCategorical_data.params.ntimepoints)) = ...
                [DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean - ...
                 sqrt(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).var);...
                 DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean + ...
                 sqrt(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).var)];

            DIVA_AuditoryCortexCategorical_data.params.match=match-1;

            % Copy auditory target representation to AuditoryCortex
            DIVA('AuditoryCortex','WeightsFromTargets',temp);
            disp(['DIVA_AuditoryCortexCategorical: Created new phonemic target: ',...
                  DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).label]);

          else, % existing phonemic target
                % Update Mean target value
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean = ...
                (DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples*...
                 DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean+SoundSegment)/...
                (DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples+1);

            % Update Variance
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).var = ...
                (DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples*...
                 DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).var+...
                 repmat([400;2500;10000],1,size(SoundSegment,2)))/...
                (DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples+1);

            % Update Velocity
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).velocity = ...
                (DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples*...
                 DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).velocity+vel)/...
                (DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples+1);

            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples = ...
                DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).nsamples+1;

            % updates AuditoryCortex targets too
            temp = DIVA('AuditoryCortex','WeightsFromTargets');

            temp(1:2*size(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean,1),...
                 DIVA_AuditoryCortexCategorical_data.params.ntimepoints*(match-1)+...
                 (1:DIVA_AuditoryCortexCategorical_data.params.ntimepoints)) = ...
                [DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean - ...
                 sqrt(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).var);...
                 DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).mean + ...
                 sqrt(DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).var)];

            DIVA_AuditoryCortexCategorical_data.params.match=match-1;
            % Copy auditory target representation to AuditoryCortex
            DIVA('AuditoryCortex','WeightsFromTargets',temp);
            disp(['DIVA_AuditoryCortexCategorical: Updated phonemic target ',...
                  DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).label]);
          end

        else
          disp('Phoneme Label not specified');
        end
      end
      DIVA_AuditoryCortexCategorical_data.params.current.SoundSegment=[];
      DIVA_AuditoryCortexCategorical_data.params.current.PhonemeLabel=[];
    end

    if ~nargout, % indicate activity in ModelStatePlot
      DIVA('ModelStatePlot','AuditoryCortexCategorical','sound');
    end

    %*********************** END SOUND ***************************%

   case 'label',
    DIVA_AuditoryCortexCategorical_data.params.current.PhonemeLabel=varargin{indexargin+1};
    %*********************** END LABEL ***************************%

   case 'target',% Sends target commands to SoundMap

    % Original--JSB 16-10-2006
    %  if iscell(varargin{indexargin+1}),
    %    targetlabel=varargin{indexargin+1}{1};
    %  else,
    %    targetlabel=varargin{indexargin+1};
    %    vel = 1;
    %  end
    %  if iscell(varargin{indexargin+1}) & length(varargin{indexargin+1})>1,
    %    vel=varargin{indexargin+1}{2};
    %  end % velocity

    % New for GoDIVA interface
    out=[];
    if(~iscell(varargin{indexargin+1}))
      tempQueue{1}=varargin{indexargin+1};
    else
      tempQueue=varargin{indexargin+1};
    end
    vel=1;
    m=1;
    delay(1)=0;
    for n=1:length(tempQueue)
      if(ischar(tempQueue{n}))
        targetQueue{m}=tempQueue{n};
        m=m+1;
        delay(m)=0;
      else
        delay(m)=delay(m)+tempQueue{n};
      end
    end

    for n=1:length(targetQueue)
      if(ischar(targetQueue{n}))
        targetlabel=targetQueue{n};
      end
      
      if(n>1)
        delay(n)=delay(n)+length(out)*DIVA('TimeStep');
      end
      match = strmatch(targetlabel,...
                       {DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(:).label},...
                       'exact');
      % Set current auditory target
      DIVA_AuditoryCortexCategorical_data.params.match=match-1;
      if isempty(match),
        warning('DIVA_AuditoryCortexCategorical: Non-existing target'); %#ok<WNTAG>
      end

      % Velocity scaling, hardcoded to normal velocity (see vel = ... above)
      velthis = vel*DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).velocity;

      if ~velthis, %#ok<BDLGI> % static target
        targetindexthis = DIVA_AuditoryCortexCategorical_data.params.ntimepoints/2*...
            ones(1,ceil(vel/DIVA_AuditoryCortexCategorical_data.params.TimeStep));
        idx=(match-1)*DIVA_AuditoryCortexCategorical_data.params.ntimepoints + targetindexthis;
        % target indexes incorporate the phonemic target 'match'
      else,        % dynamic target
        [nill,targetindexthis] = equalstep(cumsum([0,1./velthis]),...
                                           DIVA_AuditoryCortexCategorical_data.params.TimeStep);
        % Target indexes
        idx=(match-1)*DIVA_AuditoryCortexCategorical_data.params.ntimepoints + targetindexthis;
        % target indexes incorporate the phonemic target 'match'
      end
      out=idx;
      if ~nargout
        match=DIVA_AuditoryCortexCategorical_data.params.match+1;
        DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).ntrained=...
            DIVA_AuditoryCortexCategorical_data.params.AuditoryTargets(match).ntrained+1;
        DIVA('SoundMap','target',out,...
             sum(delay(1:n))+DIVA_AuditoryCortexCategorical_data.params.delayToSoundMap);
        DIVA('ModelStatePlot','AuditoryCortexCategorical','target');
      end
    end
    %*************************** END TARGET ***************************%

    % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

   otherwise, % Handle read/write parameters
    if isfield(DIVA_AuditoryCortexCategorical_data.params,varargin{indexargin}),
      if indexargin==nargin,
        out=DIVA_AuditoryCortexCategorical_data.params.(varargin{indexargin});
      else,
        DIVA_AuditoryCortexCategorical_data.params.(varargin{indexargin})=varargin{indexargin+1};
      end
    else,
      warning('DIVA_AuditoryCortexCategorical: wrong argument');
    end
  end
end

