function out=DIVA_GUI(varargin)
% DIVA_GUI DIVA Graphical User Interface
%
% The DIVA_GUI Module provides a standard user interface for the DIVA model.  It
% allows the user to run DIVA simulations and observe changes in acoustic properties
% due to VocalTract manipulation.  The various DIVA components send their output streams
% to the GUI module for graphical display to the user.  In addition the user can save
% DIVA productions as sound WAV files or Quicktime movie MOV files.  The movie files
% consist of interlaced audio and video streams.  
%
% DIVA_GUI('init' [,SessionName]);          Initializes the module
% DIVA_GUI('save' [,SessionName] );         Saves state
% DIVA_GUI('exit');                         Exits the module (without saving)
% DIVA_GUI(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%

% Dependencies:
% fcn/DIVA_GUI_callbacks
% fcn/DIVA_GUI_toolbox
% fcn

% 2007-03-26: (JSB) Added Experimental Control for simulation of noisy system and
% perturbation studies

% TODO % Perturbation capability

  out=[];
  global DIVA_GUI_data

  for indexargin=1:2:nargin,
    switch(varargin{indexargin}),
     case 'init',
      if strcmp(DIVA('status'),'off'),
        DIVA;
      else,
        SessionFolder=strcat(DIVA('SessionFolder'),filesep);
        if nargin<indexargin+1 || isempty(varargin{indexargin+1}),
          initfile='';
        else,
          initfile=[SessionFolder,'Session_',varargin{indexargin+1},...
                    filesep,mfilename,'.mat'];
        end
        if isempty(initfile) || isempty(dir(initfile)),
          disp([mfilename, ' : Defining new session...']);
          % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          DIVA_GUI_data.params=struct(...
              'act_plot',1,...      % Current active plot in 'other_weights'
              'train_sound',0,...   % GUI set to train a new sound (0: no, 1: train)
              'nTrain',2,...        % Number of training trials
              'current_file',[],... % Current training sound file
              'timepoint',0,...     % default timepoint for plot alignment
              'AutoMovie',1,...     % Automatically create a movie after training
              'duration',500);	    % duration of VT sound generation
        else,
          data=load(initfile,'-mat');
          DIVA_GUI_data.params=data.params;
        end
        % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        h=findobj('name','DIVA GUI');
        if isempty(h),
          DIVA_GUI_data.private.figurehandle=...
              figure('name','DIVA GUI','numbertitle', ...
                     'off','units','norm','menu','none', ...
                     'doublebuffer', 'on','color',[0.5 0.5 0.5],...
                     'PaperOrientation','landscape',...
                     'deletefcn','DIVA(''removemodule'',''GUI'');');
          ss=get(0,'ScreenSize');
          if(ss(3)>1400),
            set(DIVA_GUI_data.private.figurehandle,...
                'position',[.1 .1 .45 .75]);
          else
            set(DIVA_GUI_data.private.figurehandle,...
                'position',[.1 .1 .75 .75])
          end
          
          % don't know if I need handles for all this...not ever changing
          axes('position',[0 0 1 1],'xlim',[0 1],'ylim',[0 1],...
               'color',[0.5 0.5 0.5]);
          line([.32 .32],[0 1],'Color',[.7 .7 .7],'LineWidth',3);
          line([.65 .65],[0 1],'Color',[.7 .7 .7],'LineWidth',3);
          axis off;

          %---------------------DATA---------------------------------%

          DIVA_GUI_data.private.data.artics=...
              {'jh','tbp','tbs','ttp','lp','lh','ulp','llp'};
          DIVA_GUI_data.private.data.artic_label=...
              {'Jaw Height','Tongue Body Position',...
               'Tongue Body Shape','Tongue Tip Position','Lip Protrusion',...
               'Larynx Height','Upper Lip Position','Lower Lip Position'};

          DIVA_GUI_data.private.data.glot_label={'Ag0','AgP','F0'};
          DIVA_GUI_data.private.data.formant_label={'F1','F2','F3'};
          DIVA_GUI_data.private.data.showfmt=0;
          DIVA_GUI_data.private.data.diva_reps=...
              {'Motor Activity','Auditory Signal', 'Somatosensory Activity',...
               'Motor Weights','Somatosensory Weights', ...
               'Acoustic Error', 'Somatosensory Error'};

          %%%%%%%%%%%%%%%%%%%%%%%%% Menu Items %%%%%%%%%%%%%%%%%%%%%%%%%%

          %---------------File-----------------%

          DIVA_GUI_data.private.guimenu.file.main=...
              uimenu('label','File', 'tag','file', ...
                     'parent',DIVA_GUI_data.private.figurehandle,...
                     'position',1);
          DIVA_GUI_data.private.guimenu.file.new=...
              uimenu('label','New Session','tag','new',...
                     'parent',DIVA_GUI_data.private.guimenu.file.main,...
                     'position',1,'Callback',@gui_file_menu_cbk);
          DIVA_GUI_data.private.guimenu.file.load=...
              uimenu('label','Load Session','tag','load',...
                     'parent',DIVA_GUI_data.private.guimenu.file.main,...
                     'position',2,'Callback',@gui_file_menu_cbk);
          DIVA_GUI_data.private.guimenu.file.save=...
              uimenu('label','Save Session','tag','save',...
                     'parent',DIVA_GUI_data.private.guimenu.file.main,...
                     'position',3,'Callback',@gui_file_menu_cbk,...
                     'enable','on');

          DIVA_GUI_data.private.guimenu.export.main=...
              uimenu('label','Export','tag','export',...
                     'parent',DIVA_GUI_data.private.guimenu.file.main,...
                     'position',4,'separator','on');

          DIVA_GUI_data.private.guimenu.file.export.sound=...
              uimenu('label','Sound','tag','sound',...
                     'parent',DIVA_GUI_data.private.guimenu.export.main,...
                     'position',1,'callback',@gui_file_menu_cbk,...
                     'enable','off');

          DIVA_GUI_data.private.guimenu.file.export.movie=...
              uimenu('label','Movie','tag','movie',...
                     'parent',DIVA_GUI_data.private.guimenu.export.main,...
                     'position',2,'callback',@gui_file_menu_cbk,...
                     'enable','off');

          % Export plots

          DIVA_GUI_data.private.guimenu.file.screenshot=...
              uimenu('label','Screenshot','tag','screenshot',...
                     'parent',DIVA_GUI_data.private.guimenu.file.main,...
                     'position',5,'Callback',...
                     {@gui_file_menu_cbk,DIVA_GUI_data.private.figurehandle});
          DIVA_GUI_data.private.guimenu.file.exit=...
              uimenu('label','Exit','tag','exit',...
                     'parent',DIVA_GUI_data.private.guimenu.file.main,...
                     'separator','on',...
                     'position',6,'Callback',@gui_file_menu_cbk);

          %-----------Parameters------------------%

          DIVA_GUI_data.private.guimenu.parameters.main=...
              uimenu('label','Parameters','tag','param',...
                     'parent',DIVA_GUI_data.private.figurehandle,...
                     'position',2);

          %-----------------DIVA params-----------%

          DIVA_GUI_data.private.guimenu.parameters.diva.main=...
              uimenu('label','DIVA Control','tag','diva_params',...
                     'parent',DIVA_GUI_data.private.guimenu.parameters.main,...
                     'position',1);

          DIVA_GUI_data.private.guimenu.parameters.diva.timestep=...
              uimenu('label','Time Step','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.diva.main,...
                     'position',1,'callback',{@gui_params_menu_cbk,...
                              struct('name','Time Step','param','TimeStep',...
                                     'range',[0.001 0.005],'module','DIVA')});

          DIVA_GUI_data.private.guimenu.parameters.diva.plotstep=...
              uimenu('label','Plot Step','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.diva.main,...
                     'position',2,'callback',{@gui_params_menu_cbk,...
                              struct('name','Plot Step','param','PlotStep',...
                                     'range',[1 50],'module','DIVA')});

          DIVA_GUI_data.private.guimenu.parameters.diva.storagetime=...
              uimenu('label','Storage Time','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.diva.main,...
                     'position',3,'callback',{@gui_params_menu_cbk,...
                              struct('name','Storage Time','param','StorageTime',...
                                     'range',[1 25],'module','DIVA')});
          
          DIVA_GUI_data.private.guimenu.parameters.diva.blockmode=...
              uimenu('label','Block Mode','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.diva.main,...
                     'position',4,'callback',{@gui_params_menu_cbk,...
                              struct('name','Block Mode','param','BlockMode',...
                                     'range',[0 1],'module','DIVA')});
          if(DIVA('BlockMode')),
            set(DIVA_GUI_data.private.guimenu.parameters.diva.blockmode,'checked','on');
          else
            set(DIVA_GUI_data.private.guimenu.parameters.diva.blockmode,'checked','off');
          end
          
          %-------------------Motor Cortex params------------------%

          DIVA_GUI_data.private.guimenu.parameters.motor.main=...
              uimenu('label','Motor','tag','motor_params',...
                     'parent',DIVA_GUI_data.private.guimenu.parameters.main,...
                     'position',2);

          DIVA_GUI_data.private.guimenu.parameters.motor.eps_motor=...
              uimenu('label','Integration Rate','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',1,'callback',{@gui_params_menu_cbk,...
                              struct('name','eps_motor','param','eps_motor',...
                                     'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.eps_decay=...
              uimenu('label','Decay Rate','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',2,'callback',{@gui_params_menu_cbk,...
                              struct('name','eps_decay','param','eps_decay',...
                                     'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.weight_ff=...
              uimenu('label','Feedforward Weight','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',3,'separator','on',...
                     'callback',{@gui_params_menu_cbk,...
                              struct('name','Feedforward Weight',...
                                     'param','WeightFeedForward',...
                                     'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.weight_fb=...
              uimenu('label','Feedback Weight','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',4,'separator','off',...
                     'callback',{@gui_params_menu_cbk,...
                              struct('name','Feedback Weight',...
                                     'param','WeightFeedBack',...
                                     'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.weight_fba=...
              uimenu('label','Auditory Feedback Weight','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',5,'separator','off',...
                     'callback',{@gui_params_menu_cbk,...
                              struct('name','Auditory Feedback Weight',...
                                     'param','WeightFeedBack_sound',...
                                     'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.weight_fbs=...
              uimenu('label','Somatosensory Feedback Weight','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',6,'separator','off','callback',...
                     {@gui_params_menu_cbk,...
                      struct('name','Somatosensory Feedback Weight',...
                             'param','WeightFeedBack_somatosensory',...
                             'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.weight_learn=...
              uimenu('label','Learning Weight','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',7,'separator','on','callback',...
                     {@gui_params_menu_cbk,...
                      struct('name','Learning Weight',...
                             'param','WeightLearn',...
                             'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.weight_learn_a=...
              uimenu('label','Learning Weight (auditory)','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',8,'separator','off','callback',...
                     {@gui_params_menu_cbk,...
                      struct('name','Learning Weight (auditory)',...
                             'param','WeightLearn_sound',...
                             'range',[0 1],'module','MotorCortex')});

          DIVA_GUI_data.private.guimenu.parameters.motor.weight_learn_s=...
              uimenu('label','Learning Weight (somatosensory)','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.motor.main,...
                     'position',9,'separator','off','callback',...
                     {@gui_params_menu_cbk,...
                      struct('name','Learning Weight (somatosensory)',...
                             'param','WeightLearn_somatosensory',...
                             'range',[0 1],'module','MotorCortex')});

          %-------------------Somatosensory Cortex params------------------%

          DIVA_GUI_data.private.guimenu.parameters.somato.main=...
              uimenu('label','Somatosensory','tag','somato_params',...
                     'parent',DIVA_GUI_data.private.guimenu.parameters.main,...
                     'position',3);

          DIVA_GUI_data.private.guimenu.parameters.somato.eps_learn=...
              uimenu('label','Learning Rate','parent',...
                     DIVA_GUI_data.private.guimenu.parameters.somato.main,...
                     'position',1,'callback',...
                     {@gui_params_menu_cbk,...
                      struct('name','eps_learn',...
                             'param','eps_learn',...
                             'range',[0 1],'module','SomatosensoryCortex')});

          %-------------------All params------------------%

          DIVA_GUI_data.private.guimenu.parameters.all=...
              uimenu('label','Edit All','tag','all_params',...
                     'parent',DIVA_GUI_data.private.guimenu.parameters.main,...
                     'position',4,'separator','on','callback',...
                     @gui_all_params_cbk);

          %-------------------Plot Glottal params------------------%

          DIVA_GUI_data.private.guimenu.parameters.glot=...
              uimenu('label','View Glottal Trajectory','tag','glot_params',...
                     'parent',DIVA_GUI_data.private.guimenu.parameters.main,...
                     'position',5,'callback',@gui_params_menu_cbk);

          %---------------Target Source---------------%

          DIVA_GUI_data.private.guimenu.target.main=...
              uimenu('label','Target source','tag','target_menu',...
                     'parent',DIVA_GUI_data.private.figurehandle,...
                     'position',3);
          DIVA_GUI_data.private.guimenu.target.file=...
              uimenu('label','Auditory Target from file','tag','file',...
                     'parent',DIVA_GUI_data.private.guimenu.target.main,...
                     'position',1,'enable','on','Callback',...
                     @gui_targ_menu_cbk);
          DIVA_GUI_data.private.guimenu.target.synth=...
              uimenu('label','Synthesize Auditory Target','tag','synth',...
                     'parent',DIVA_GUI_data.private.guimenu.target.main,...
                     'position',2','enable','on','Callback',...
                     @gui_targ_menu_cbk);
          DIVA_GUI_data.private.guimenu.target.mic=...
              uimenu('label','Auditory Target from microphone','tag','mic',...
                     'parent',DIVA_GUI_data.private.guimenu.target.main,...
                     'position',3,'enable','off','Callback',...
                     @gui_targ_menu_cbk);

          %-------------Preferences-------------------%

          DIVA_GUI_data.private.guimenu.prefs.main=...
              uimenu('label','Preferences','tag','prefs_menu',...
                     'parent',DIVA_GUI_data.private.figurehandle,...
                     'position',4);

          DIVA_GUI_data.private.guimenu.prefs.ntrain=...
              uimenu('label','Number of Training Trials',...
                     'tag','nTrain',...
                     'parent',DIVA_GUI_data.private.guimenu.prefs.main,...
                     'position',1,'callback',@gui_prefs_menu_cbk);

          DIVA_GUI_data.private.guimenu.prefs.createMovie=...
              uimenu('label','Auto Movie Playback','tag','movie',...
                     'parent',DIVA_GUI_data.private.guimenu.prefs.main,...
                     'position',2,'callback',@gui_prefs_menu_cbk);
          if(DIVA_GUI_data.params.AutoMovie==1),
            set(DIVA_GUI_data.private.guimenu.prefs.createMovie,'checked','on');
          else
            set(DIVA_GUI_data.private.guimenu.prefs.createMovie,'checked','off');
          end

          %-------------Experimental Control-------------------------%          
          
          DIVA_GUI_data.private.guimenu.expctrl.main = ...
              uimenu('label','Experimental Control','tag','exp_ctrl',...
                     'parent',DIVA_GUI_data.private.figurehandle,...
                     'position',5);
          DIVA_GUI_data.private.guimenu.expctrl.noise = ...
               uimenu('label','Model Noise','tag','noise',...
                      'parent',DIVA_GUI_data.private.guimenu.expctrl.main,...
                      'callback',@gui_expctrl_noise,'position',1);
          DIVA_GUI_data.private.guimenu.expctrl.perturb = ...
               uimenu('label','Perturbation','tag','perturb',...
                      'parent',DIVA_GUI_data.private.guimenu.expctrl.main,...
                      'position',2,'enable','off');
             
          %-------------About-------------------------%

          DIVA_GUI_data.private.guimenu.help.main=...
              uimenu('label','Help','tag','help_menu',...
                     'parent',DIVA_GUI_data.private.figurehandle,...
                     'position',6);
          DIVA_GUI_data.private.guimenu.help.diva=...
              uimenu('label','DIVA Help','tag','diva',...
                     'parent',DIVA_GUI_data.private.guimenu.help.main,...
                     'position',1,'Callback',@gui_help_menu_cbk);
          DIVA_GUI_data.private.guimenu.help.readme=...
              uimenu('label','Readme','tag','readme',...
                     'parent',DIVA_GUI_data.private.guimenu.help.main,...
                     'position',2,'Callback',@gui_help_menu_cbk);
          DIVA_GUI_data.private.guimenu.help.depgraph=...
              uimenu('label','Dependency Graph','tag','depgraph',...
                     'parent',DIVA_GUI_data.private.guimenu.help.main,...
                     'position',3,'Callback',@gui_help_menu_cbk);

          %%%%%%%%%%%%%%%%%%%% Model Section %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

          DIVA_GUI_data.private.model.label.title = ...
              uicontrol('Style','text','parent',DIVA_GUI_data.private.figurehandle,...
                        'String','DIVA Model Control','units','norm',...
                        'fontunits','norm','fontsize',0.6,...
                        'position',[0.04 0.95 0.22 0.04],...
                        'backgroundcolor',[0.5 0.5 0.5]);

          %--------------------Learned Sounds-----------------------------%

          DIVA_GUI_data.private.model.menu.learned=...
              uicontrol('Style','popupmenu','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'tag','menu',...
                        'String','----','Enable','off','units',...
                        'norm','position',[0.080 0.86 0.16 0.05],...
                        'FontUnits','norm','FontSize',0.3,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'callback',@gui_model_cbk);

          DIVA_GUI_data.private.model.label.learned=...
              uicontrol('Style','text','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'String','Stored Auditory Targets','Enable','on',...
                        'units','norm','position',[0.06 0.93 0.2 0.02],...
                        'FontUnits','norm','FontSize',0.9,...
                        'BackgroundColor',[0.5 0.5 0.5]);

          DIVA_GUI_data.private.model.button.train=...
              uicontrol('Style','pushbutton','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'String','Practice Sound','Enable','off',...
                        'tag','train',...
                        'units','norm','position',[0.080 0.85 0.085 0.03],...
                        'FontUnits','norm','FontSize',0.45,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'Callback',@gui_model_cbk);

          DIVA_GUI_data.private.model.button.reset=...
              uicontrol('Style','pushbutton','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'String','Reset Weights','Enable','off',...
                        'tag','reset',...
                        'units','norm','position',[0.16 0.85 0.085 0.03],...
                        'FontUnits','norm','FontSize',0.45,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'Callback',@gui_model_cbk);

          DIVA_GUI_data.private.model.button.play=...
              uicontrol('Style','pushbutton','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'tag','play',...
                        'String','Play Stored Video','Enable','off','units',...
                        'norm','position',[0.080 0.82 0.085 0.03],...
                        'FontUnits','norm','FontSize',0.45,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'Callback',@gui_model_cbk);

          DIVA_GUI_data.private.model.button.produce=...
              uicontrol('Style','pushbutton','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'tag','produce',...
                        'String','Produce Sound','Enable','off','units',...
                        'norm','position',[0.16 0.82 0.085 0.03],...
                        'FontUnits','norm','FontSize',0.45,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'Callback',@gui_model_cbk);

          %-----------------Target Specification---------------------%

          DIVA_GUI_data.private.model.label.target_file=...
              uicontrol('Style','text','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'String',[],'Enable','on',...
                        'units','norm','position',[0.06 0.75 0.1 0.03],...
                        'FontUnits','norm','FontSize',0.5,...
                        'BackgroundColor',[0.7 0.7 0.7],...
                        'HorizontalAlignment','center');

          DIVA_GUI_data.private.model.button.target_file=...
              uicontrol('Style','pushbutton','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'tag','browse',...
                        'String','Browse','Enable','on',...
                        'units','norm','position',[0.16 0.75 0.075 0.03],...
                        'FontUnits','norm','FontSize',0.4,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'Callback',@gui_model_cbk);

          if(exist(['data',filesep,'mic1_new.jpeg'],'file'))
            mic=double(imread(['data',filesep,'mic1_new.jpeg']))/255;
            mic_str=[];
            bColor=[0.5 0.5 0.5];
          else
            mic=[];
            mic_str='mic';
            bColor=[0.5 0.5 0.5];
          end
          DIVA_GUI_data.private.model.button.target_mic=...
              uicontrol('Style','pushbutton','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'CData',mic,'tag','mic',...
                        'String',mic_str,'Enable','off',...
                        'units','norm','position',[0.235 0.75 0.025 0.03],...
                        'FontUnits','norm','FontSize',0.3,...
                        'BackgroundColor',bColor);

          DIVA_GUI_data.private.model.label.target=...
              uicontrol('Style','text','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'String','Speech Sound Target',...
                        'Enable','on','units','norm',...
                        'position',[0.06 0.78 0.2 0.02],...
                        'FontUnits','norm','FontSize',0.7,...
                        'BackgroundColor',[0.5 0.5 0.5]);

          DIVA_GUI_data.private.model.button.acquire=...
              uicontrol('Style','pushbutton','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'tag','acquire',...
                        'String','Acquire Target','Enable','off',...
                        'units','norm','position',[0.06 0.72 0.2 0.03],...
                        'FontUnits','norm','FontSize',0.5,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'Callback',@gui_model_cbk);

          %-------------------- Auditory Representation------------------%

          DIVA_GUI_data.private.model.guiaxes.auditory=...
              axes('Position',[0.05 0.45 0.22 0.22],'color','w',...
                   'parent',DIVA_GUI_data.private.figurehandle,...
                   'visible','on','xlim',...
                   [0 DIVA('AuditoryCortexCategorical','ntimepoints')],...
                   'ylim',[0 3500],...
                   'buttondownfcn',@gui_plot_align);

          nFormants=DIVA('AuditoryCortex','nFormants');
          DIVA_GUI_data.private.model.traces.auditory = ...
              zeros(nFormants,1);
          DIVA_GUI_data.private.model.target.auditory = ...
              zeros(nFormants*2,1);
          for n=1:nFormants,
            DIVA_GUI_data.private.model.traces.auditory(n) = ...
                line([0 0],[0 0],'color','r','visible','off',...
                     'ButtonDownFcn',@gui_plot_align);
            DIVA_GUI_data.private.model.target.auditory(n) = ...
                line([0 0],[0 0],'color','k','visible','off',...
                     'ButtonDownFcn',@gui_plot_align);
            DIVA_GUI_data.private.model.target.auditory(n+nFormants) = ...
                line([0 0],[0 0],'color','k','visible','off',...
                     'ButtonDownFcn',@gui_plot_align);
          end
          DIVA_GUI_data.private.model.timeline.auditory = ...
              line([0 0],[0 0],'color','k','visible','off',...
                   'parent',DIVA_GUI_data.private.model.guiaxes.auditory,...
                   'buttondownfcn',@gui_plot_align);

          DIVA_GUI_data.private.model.title.auditory=...
              title('Auditory Representation from Target','parent',...
                    DIVA_GUI_data.private.model.guiaxes.auditory,...
                    'visible','on');
          axis xy;

          %--------------Other Model Representations-------------------%

          cco=[0 0 1;1 0 0;0 .5 0;0 0 0];
          rco=rand(7,3);
          rco=rco-repmat(min(rco,[],1),7,1);
          rco=rco*diag(1./max(rco,[],1));
          nco=[cco;rco];

          DIVA_GUI_data.private.model.guiaxes.other=...
              axes('Position',[0.05 0.1 0.22 0.22],'color','w',...
                   'parent',DIVA_GUI_data.private.figurehandle,...
                   'visible','on','xlim',...
                   [0 DIVA('AuditoryCortexCategorical','ntimepoints')],...
                   'ylim',[0 1],'colororder',nco);

          DIVA_GUI_data.private.model.timeline.other=...
              line([0 0],[0 0],'color','k','visible','off',...
                   'buttondownfcn',@gui_plot_align);

          DIVA_GUI_data.private.model.menu.other=...
              uicontrol('style','popupmenu','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'String',DIVA_GUI_data.private.data.diva_reps,...
                        'Enable','on','units','norm',...
                        'tag','other',...
                        'position',[0.05 0.32 0.22 0.05],...
                        'FontUnits','norm','FontSize',0.3,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'Callback',@gui_model_cbk);

          maxModeDim = max([DIVA('AuditoryCortex','nFormants'),...
                            DIVA('MotorCortex','nMotor'),...
                            DIVA('SomatosensoryCortex','nSensory')]);

          DIVA_GUI_data.private.model.traces.other = zeros(maxModeDim,1);
          for n=1:maxModeDim,
            DIVA_GUI_data.private.model.traces.other(n) = ...
                line([0 0],[0 0],'visible','off',...
                     'parent',DIVA_GUI_data.private.model.guiaxes.other,...
                     'buttondownfcn',@gui_plot_align);
          end

          %%%%%%%%%%%%%% Auditory Section %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

          DIVA_GUI_data.private.auditory.label.acoustic=...
              uicontrol('style','text','units','norm',...
                        'position',[0.39 0.95 0.24 0.04],...
                        'String',sprintf('Acoustic Representations'),...
                        'parent',DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'FontUnits',...
                        'norm','FontSize',0.6);

          %---------------- Formant Representation ----------------%

          DIVA_GUI_data.private.auditory.label.formant=...
              uicontrol('style','text','units','norm',...
                        'position',[0.41 0.875 0.2 0.025],...
                        'string','Formant Values (Hz)',...
                        'parent',DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],...
                        'FontUnits','norm','FontSize',0.6);

          fmt_label=DIVA_GUI_data.private.data.formant_label;
          for i=1:length(fmt_label)
            DIVA_GUI_data.private.auditory.label.(fmt_label{i})=...
                uicontrol('style','text','units','norm',...
                          'position',[0.41+(i-1)*(.2/3) 0.855 0.05 0.015],...
                          'FontUnits','norm','FontSize',0.9,...
                          'string',fmt_label{i},'parent', ...
                          DIVA_GUI_data.private.figurehandle,...
                          'BackgroundColor',[0.5 0.5 0.5]);
          end
          for i=1:length(fmt_label)
            DIVA_GUI_data.private.auditory.edit.(fmt_label{i})=...
                uicontrol('style','edit','units','norm',...
                          'tag','fmt',...
                          'position',[0.41+(i-1)*(.2/3) 0.82 0.05 0.03],...
                          'string',0,'enable','on','parent',...
                          DIVA_GUI_data.private.figurehandle,...
                          'callback',@gui_auditory_cbk);
          end

          DIVA_GUI_data.private.auditory.button.go=...
              uicontrol('style','pushbutton','units','norm',...
                        'position',[0.41 0.78 2*.2/3+0.052 0.03],...
                        'string','GO','enable','on','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'tag','go','callback',@gui_auditory_cbk);

          % ---------------- Spectral Representation Plot ---------------%

          DIVA_GUI_data.private.auditory.guiaxes.spectral=...
              axes('Position',[0.4 0.45 0.22 0.22],'color','w',...
                   'parent',DIVA_GUI_data.private.figurehandle,...
                   'visible','on','xlim',[1 20],'ylim',[0 3500]);
          DIVA_GUI_data.private.auditory.gui_image.spectral=...
              image(zeros(257,200),'visible','off');
          title('Spectral Representation','Parent',...
                DIVA_GUI_data.private.auditory.guiaxes.spectral);
          %load 'data/specgram_cmap.mat'
          %colormap(DIVA_GUI_data.private.auditory.guiaxes.spectral,cmap);
          colormap(DIVA_GUI_data.private.auditory.guiaxes.spectral,1-gray);

          axis xy;

          DIVA_GUI_data.private.auditory.traces.f1=...
              line([1 10],...
                   [1 1],'Color','blue','LineWidth',2.5,'Visible','off');
          DIVA_GUI_data.private.auditory.traces.f2=...
              line([1 10],...
                   [1 1],'Color','green','LineWidth',2.5,'Visible','off');
          DIVA_GUI_data.private.auditory.traces.f3=...
              line([1 10],...
                   [1 1],'Color','red','LineWidth',2.5,'Visible','off');

          % -------------- Auditory Space (F1/F2) Plot ----------%

          DIVA_GUI_data.private.auditory.guiaxes.fmt_space=...
              axes('Position',[0.40 0.1 0.22 0.22],'color','w',...
                   'parent',DIVA_GUI_data.private.figurehandle,...
                   'visible','on','tag','fmt_space',...
                   'ButtonDownFcn',@gui_auditory_cbk);
          % Callback for clicking plot, move to vowels specified by click location
          au_fmt_space_init(DIVA_GUI_data.private.auditory.guiaxes.fmt_space);

          DIVA_GUI_data.private.auditory.crosshair.xline=...
              line(get(DIVA_GUI_data.private.auditory.guiaxes.fmt_space,'xlim'),...
                   ones(1,2)*mean(get(DIVA_GUI_data.private.auditory.guiaxes.fmt_space,'ylim')),...
                   'Color','black','tag','fmt_space','buttondownfcn',@gui_auditory_cbk);
          DIVA_GUI_data.private.auditory.crosshair.yline=...
              line(ones(1,2)*mean(get(DIVA_GUI_data.private.auditory.guiaxes.fmt_space,'xlim')),...
                   get(DIVA_GUI_data.private.auditory.guiaxes.fmt_space,'ylim'),...
                   'Color','black','tag','fmt_space','buttondownfcn',@gui_auditory_cbk);

          %%%%%%%%%%%%%%%%%%%%%%%%% Vocal Tract %%%%%%%%%%%%%%%%%%%%%%%%%%%%%

          DIVA_GUI_data.private.vocaltract.label.title = ...
              uicontrol('Style','text','parent',DIVA_GUI_data.private.figurehandle,...
                        'String','Vocal Tract Control','units','norm',...
                        'fontunits','norm','fontsize',0.6,...
                        'position',[0.72 0.95 0.22 0.04],...
                        'backgroundcolor',[0.5 0.5 0.5]);

          
          artics=DIVA_GUI_data.private.data.artics;
          artic_label=DIVA_GUI_data.private.data.artic_label;

          DIVA_GUI_data.private.vocaltract.label.artic=...
              uicontrol('style','text','units','norm',...
                        'position',[0.72 0.89 0.1 0.05],...
                        'String',sprintf('Articulatory Configuration'),...
                        'parent',DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'FontUnits',...
                        'norm','FontSize',0.3);

          %--------------- VT Control -----------------------%

          DIVA_GUI_data.private.vocaltract.button.random=...
              uicontrol('style','pushbutton','units','norm',...
                        'position',[0.89 0.92 0.05 0.03],...
                        'String','Random','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'Enable','on',...
                        'FontUnits','norm','FontSize',0.45,...
                        'tag','random',...
                        'Callback',{@gui_vocaltract_cbk,...
                              DIVA_GUI_data.private.data.artics});

          DIVA_GUI_data.private.vocaltract.button.reset=...
              uicontrol('style','pushbutton','units','norm',...
                        'position',[0.89 0.89 0.05 0.03],...
                        'String','Reset','parent',...
                        DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'Enable','on',...
                        'FontUnits','norm','FontSize',0.5,...
                        'tag','reset',...
                        'Callback',{@gui_vocaltract_cbk,...
                              {DIVA_GUI_data.private.data.artics,...
                              DIVA_GUI_data.private.data.glot_label}});

          maxVal=DIVA('VocalTract','MaedaSpread');
          minVal=-DIVA('VocalTract','MaedaSpread');
          sstep=[0.01 0.1]./(2*DIVA('VocalTract','MaedaSpread'));
          for i=1:8
            DIVA_GUI_data.private.vocaltract.slider.(artics{i})=...
                uicontrol('style','slider','value',0.00,...
                          'units','norm','tag','articslider',...
                          'position',[0.67+0.2*(1/3)+0.005 0.84-(.045*(i-1)) ...
                                0.2*(2/3)-0.005 0.015],...
                          'parent',DIVA_GUI_data.private.figurehandle,...
                          'enable','on','Callback',{@gui_vocaltract_cbk},...
                          'UserData',artics{i},'min',minVal,'max',maxVal,...
                          'SliderStep',sstep);

            DIVA_GUI_data.private.vocaltract.edit.(artics{i})=...
                uicontrol('style','edit','string','0.00',...
                          'units','norm','tag','articedit',...
                          'position',[0.89 0.845-(0.045*(i-1)) 0.05 0.02],...
                          'parent',DIVA_GUI_data.private.figurehandle,...
                          'enable','on','FontUnits','norm','FontSize',0.6,...
                          'Callback',@gui_vocaltract_cbk,'UserData',artics{i});

            sl_hlist=[DIVA_GUI_data.private.vocaltract.slider.(artics{i}),...
                      DIVA_GUI_data.private.vocaltract.edit.(artics{i})];
            align(sl_hlist,'none','middle');

            numspace=findstr(' ',artic_label{i});
            if(length(artic_label{i})>=14)
              bottom=0.845-(0.045*(i-1))-0.01;
              height=0.04;
              fsize=0.7/2;
              lbl=sprintf('%s\n%s',artic_label{i}(1:numspace(end)-1),...
                          artic_label{i}(numspace(end)+1:end));
            else
              bottom=0.845-(0.045*(i-1));
              height=0.02;
              fsize=0.7;
              lbl=artic_label{i};
            end
            DIVA_GUI_data.private.vocaltract.label.(artics{i})=...
                uicontrol('style','text','units','norm',...
                          'position',[0.67 bottom 0.2*(1/3) height],...
                          'String',lbl,'enable','on',...
                          'parent',DIVA_GUI_data.private.figurehandle,...
                          'HorizontalAlignment','left',...
                          'BackgroundColor',[0.5 0.5 0.5],'FontUnits','norm',...
                          'FontSize',fsize);
          end % Articulator param control loop

          glot_label=DIVA_GUI_data.private.data.glot_label;
          for i=1:3
            if(i==3)
              max_sl=200;min_sl=0;
              sstep=[1 10]./(max_sl-min_sl);
              startval='100';
              unitstr='Hz';
            else
              if(strcmp(glot_label{i},'Ag0'))
                max_sl=1;
                min_sl=0;
                sstep=[0.01 0.1]./(max_sl-min_sl);
                startval='0.01';
                unitstr='';
              else
                max_sl=1;
                min_sl=0;
                sstep=[0.01 0.1]./(max_sl-min_sl);
                startval='0.2';
                unitstr='';
              end
            end
            DIVA_GUI_data.private.vocaltract.slider.(glot_label{i})=...
                uicontrol('style','slider','min',min_sl,'max',max_sl,...
                          'units','norm','tag','glotslider',...
                          'position',[0.72 0.47-(0.03*(i-1)) 0.15 0.013],...
                          'parent',DIVA_GUI_data.private.figurehandle,...
                          'enable','on','SliderStep',sstep,...
                          'UserData',glot_label{i},'value',str2double(startval),...
                          'Callback',@gui_vocaltract_cbk);

            DIVA_GUI_data.private.vocaltract.edit.(glot_label{i})=...
                uicontrol('style','edit','string',startval,...
                          'units','norm','tag','glotedit',...
                          'position',[0.89 0.47-(0.03*(i-1)) 0.05 0.02],...
                          'parent',DIVA_GUI_data.private.figurehandle,...
                          'enable','on','FontUnits','norm','FontSize',0.6,...
                          'UserData',struct('max',max_sl,'min',min_sl,...
                                            'name',glot_label{i}),...
                          'callback',@gui_vocaltract_cbk);

            sl_hlist=[DIVA_GUI_data.private.vocaltract.slider.(glot_label{i}),...
                      DIVA_GUI_data.private.vocaltract.edit.(glot_label{i})];
            align(sl_hlist,'none','middle');

            if(strcmp(glot_label{i},'F0'))
              lbl='Mean F0';
            else
              lbl=glot_label{i};
            end
            
            DIVA_GUI_data.private.vocaltract.label.(glot_label{i})=...
                uicontrol('style','text','units','norm',...
                          'position',[0.67 0.47-(0.03*(i-1)) 0.05 0.02],...
                          'String',lbl,'enable','on',...
                          'parent',DIVA_GUI_data.private.figurehandle,...
                          'HorizontalAlignment','left',...
                          'BackgroundColor',[0.5 0.5 0.5],'FontUnits','norm',...
                          'FontSize',0.6);

            DIVA_GUI_data.private.vocaltract.unitlabel.(glot_label{i})=...
                uicontrol('style','text','units','norm',...
                          'position',[0.94 0.47-(0.03*(i-1)) 0.05 0.02],...
                          'String',unitstr,'enable','on',...
                          'parent',DIVA_GUI_data.private.figurehandle,...
                          'FontUnits','norm','FontSize',0.6,...
                          'HorizontalAlignment','left',...
                          'BackgroundColor',[0.5 0.5 0.5]);
          end % Glottis param control loop

          %----------------Display VT ---------------------%%

          DIVA_GUI_data.private.vocaltract.guiaxes.vtplot=...
              axes('Position',[0.67 0.1 0.2 0.3],'color','w',...
                   'parent',DIVA_GUI_data.private.figurehandle,...
                   'visible','on');
          axis off;
          imagesc([]);
          set(DIVA_GUI_data.private.vocaltract.guiaxes.vtplot,...
              'xtick',[],'xticklabel',[],'ytick',[],'yticklabel',[]);
          
          DIVA_GUI_data.private.vocaltract.edit.duration=...
              uicontrol('style','edit','units','norm',...
                        'position',[0.89 0.24 0.05 0.03],...
                        'String','500','tag','duration',...
                        'parent',DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'enable','on',...
                        'FontUnits','norm','FontSize',0.4,...
                        'callback',@gui_vocaltract_cbk);
          DIVA_GUI_data.private.vocaltract.label.duration=...
              uicontrol('style','text','units','norm',...
                        'position',[0.94,0.245 0.05 0.02],...
                        'String','ms','tag','duration',...
                        'parent',DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'enable','on',...
                        'FontUnits','norm','FontSize',0.9,...
                        'HorizontalAlignment','left');
                              
          DIVA_GUI_data.private.vocaltract.button.generate=...
              uicontrol('style','pushbutton','units','norm',...
                        'position',[0.89 0.27 0.10 0.03],...
                        'String','Generate Sound','tag','generate',...
                        'parent',DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'Enable','on',...
                        'FontUnits','norm','FontSize',0.5,...
                        'Callback',{@gui_vocaltract_cbk,...
                              {DIVA_GUI_data.private.data.artics,...
                              DIVA_GUI_data.private.data.glot_label}});

          DIVA_GUI_data.private.vocaltract.button.learn=...
              uicontrol('style','pushbutton','units','norm',...
                        'position',[0.89 0.30 0.1 0.03],...
                        'String',sprintf('Store Target'),...
                        'tag','learn',...
                        'parent',DIVA_GUI_data.private.figurehandle,...
                        'BackgroundColor',[0.5 0.5 0.5],'Enable','on',...
                        'FontUnits','norm','FontSize',0.5,...
                        'Callback',{@gui_vocaltract_cbk,...
                              {DIVA_GUI_data.private.data.artics,...
                              DIVA_GUI_data.private.data.glot_label}});
        end

        DIVA_GUI('ACC_AuditoryTargets',...
                 DIVA('AuditoryCortexCategorical','AuditoryTargets'));        
        DIVA_GUI('VT_Configuration',DIVA('VocalTract','current'));
        artic=DIVA_GUI('artic_slider',[]);
        temp=DIVA('VocalTract','motor',artic);
        DIVA_GUI('FmtSpace_crosshairs',temp{1});
        DIVA_GUI('FmtValues',temp{1});
        
        % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % LIST OF MODULE CHANNELS
        % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        out ={};
        % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      end

      %--------------------------------------------------------------%

     case 'AC_WeightsFromTargets',
      set(DIVA_GUI_data.private.model.guiaxes.auditory,'visible','on');

      if(strcmp(varargin{indexargin+1},'update')),
        % Plot updated auditory representation traces
        searchTarg = DIVA('AuditoryCortex','target',[],...
                          DIVA('TimeStep')*(-1000:0));
        idx = getQueueIdx(searchTarg);
        au=DIVA('AuditoryCortex','sound',[],DIVA('TimeStep')*(-idx:0));
        targ=searchTarg(1000-idx+1:end);
        [targ,au]=chompIdx(targ,au);

        % update values of each formant line graphics object
        if(~isempty(au) && (mod(length(targ),DIVA('PlotStep'))==0 || ...
                           ~any(DIVA('busy'))))
          for n=1:DIVA('AuditoryCortex','nFormants')
            set(DIVA_GUI_data.private.model.traces.auditory(n),...
                'YData',au(n,:),...
                'XData',targ,...
                'visible','on');
          end
        end
      elseif(strcmp(varargin{indexargin+1},'target'))
        % Plot auditory target regions
        au=DIVA('AuditoryCortex','WeightsFromTargets');
        if(~isempty(au))
          set(DIVA_GUI_data.private.model.button.train,'enable','on');
          set(DIVA_GUI_data.private.model.button.reset,'enable','on');
        end
        searchTarg=DIVA('AuditoryCortex','target',[],...
                        DIVA('TimeStep')*(-1000:0));
        idx=getQueueIdx(searchTarg);
        targ=searchTarg(1000-idx+1:end);

        if(~isempty(au) && DIVA('AuditoryCortexCategorical','match')~=-1)
          matchstart = DIVA('AuditoryCortexCategorical','ntimepoints')*...
              DIVA('AuditoryCortexCategorical','match')+1;
          matchend = matchstart+DIVA('AuditoryCortexCategorical','ntimepoints')-1;
          set(DIVA_GUI_data.private.model.guiaxes.auditory,...
              'Xlim',[matchstart matchend]);
          for n=1:size(au,1),
            set(DIVA_GUI_data.private.model.target.auditory(n),...
                'XData',matchstart:matchend,...
                'YData',au(n,matchstart:matchend),...
                'visible','on');
          end
        end
      end

      %--------------------------------------------------------------%

     case 'FmtSpace_crosshairs'
      f1=varargin{indexargin+1}(1);
      f2=varargin{indexargin+1}(2);

      set(DIVA_GUI_data.private.auditory.crosshair.xline,...
          'YData',[f2 f2]);

      set(DIVA_GUI_data.private.auditory.crosshair.yline,...
          'XData',[f1 f1]);

      %--------------------------------------------------------------%

     case 'FmtValues'
      fmt_label=DIVA_GUI_data.private.data.formant_label;
      if(~nargout)
        for i=1:length(fmt_label)
          set(DIVA_GUI_data.private.auditory.edit.(fmt_label{i}),...
              'string',sprintf('%d',round(varargin{indexargin+1}(i))),...
              'value',varargin{indexargin+1}(i));
        end
      else
        for i=1:length(fmt_label)
          out(i)=get(DIVA_GUI_data.private.auditory.edit.(fmt_label{i}),...
                      'value');
        end
      end

      %--------------------------------------------------------------%

     case 'timealign'
      idx=varargin{indexargin+1};
      if(idx > 0)
        set(DIVA_GUI_data.private.model.timeline.auditory,...
            'Xdata',[idx idx],...
            'Ydata',get(DIVA_GUI_data.private.model.guiaxes.auditory,'Ylim'),...
            'visible','on');
        set(DIVA_GUI_data.private.model.timeline.other,...
            'Xdata',[idx idx],...
            'Ydata',get(DIVA_GUI_data.private.model.guiaxes.other,'Ylim'),...
            'visible','on');
        DIVA_GUI('timepoint',idx);

        searchIdx=DIVA('TimeStep')*(-1000:0);
        motor_idx_list=DIVA('MotorCortex','target',[],searchIdx);
        aud_idx_list=DIVA('AuditoryCortex','target',[],searchIdx);
        if(~isempty(motor_idx_list) && ~isempty(aud_idx_list))

          [nil motor_idx]=sort(abs(motor_idx_list-idx));
          [nil aud_idx]=sort(abs(aud_idx_list-idx));

          motor=DIVA('VocalTract','motor',[],searchIdx(motor_idx(1)));
          auditory=DIVA('AuditoryCortex','sound',[],searchIdx(motor_idx(1)));
          DIVA('GUI','FmtValues',(auditory));
          %DIVA('GUI','FmtSpace_crosshairs',mel(auditory));
          DIVA('GUI','FmtSpace_crosshairs',(auditory));

          artic=DIVA('VocalTract','motor',motor);
          artic=artic{2};
          DIVA('GUI','artic_edit',artic(1:DIVA('VocalTract','nArtic'),:));
          DIVA('GUI','artic_slider',artic(1:DIVA('VocalTract','nArtic'),:));
          DIVA('GUI','VT_Configuration',motor);
        end
      end

      %--------------------------------------------------------------%

     case 'Cochlea_plot'
      set(DIVA_GUI_data.private.auditory.guiaxes.spectral,'visible','on');
      set(DIVA_GUI_data.private.auditory.gui_image.spectral,'visible','on');

      % assume varargin{indexargin+1} is a sound waveform
      out=DIVA('Cochlea','spectrogram',varargin{indexargin+1});
      [nil,idx]=sort(abs(out{2}-3500));      
      set(DIVA_GUI_data.private.auditory.guiaxes.spectral,'xlim',[0 size(out{1},2)],...
                        'ylim',[1 idx(1)]);
      set(DIVA_GUI_data.private.auditory.gui_image.spectral,'cdata',...
                        64*abs(out{1}/max(max(abs(out{1})))),'cdatamapping','direct');
      set(DIVA_GUI_data.private.auditory.guiaxes.spectral,'ytick',20:20:length(out{2}));
      set(DIVA_GUI_data.private.auditory.guiaxes.spectral,'xtick',[]);
      set(DIVA_GUI_data.private.auditory.guiaxes.spectral,'yticklabel',...
                        round(out{2}(get(DIVA_GUI_data.private.auditory.guiaxes.spectral,'ytick'))));

      %--------------------------------------------------------------%

     case 'ACC_AuditoryTargets',
      if(~isempty(varargin{indexargin+1}))
        targmenu={};
        targmenu{1}='Select One';
        if(ischar(varargin{indexargin+1})),
          targs=DIVA('AuditoryCortexCategorical','AuditoryTargets');
          for n=1:length(targs),
            targmenu{n+1}=sprintf('%s\t%s',targs(n).label,...
                                ['(',num2str(targs(n).ntrained),')']);
          end
          startval=DIVA('AuditoryCortexCategorical','match')+2;
        else
          for n=1:length(varargin{indexargin+1})
            targmenu{n+1}=sprintf('%s\t%s',varargin{indexargin+1}(n).label,...
                                ['(',num2str(varargin{indexargin+1}(n).ntrained),')']);
          end
          startval=1;
        end
        set(DIVA_GUI_data.private.model.menu.learned,'String',targmenu);
        set(DIVA_GUI_data.private.model.menu.learned,'value',startval);
        set(DIVA_GUI_data.private.model.menu.learned,'enable','on');

        
      else
        set(DIVA_GUI_data.private.model.menu.learned,'String','--No Targets--');
        set(DIVA_GUI_data.private.model.button.play,'enable','off');
      end

      %--------------------------------------------------------------%

     case 'other_weights',
      oreps={'motor','auditory','somatosensory','motor_weight',...
             'somatosensory_weight'...
             'error_auditory','error_somatosensory'};
      if(nargin>=indexargin+1 && isempty(varargin{indexargin+1}))
        curr_plot = get(DIVA_GUI_data.private.model.menu.other,'value');
      else,
        if(DIVA_GUI_data.params.act_plot == ...
           strmatch(varargin{indexargin+1},oreps,'exact'))
          curr_plot = DIVA_GUI_data.params.act_plot;
        else
          curr_plot =[];
        end
      end
      if(~isempty(curr_plot))
        DIVA_GUI_data.params.act_plot=curr_plot;
      end
      maxModeDim=size(DIVA_GUI_data.private.model.traces.other,1);
      searchIdx=DIVA('TimeStep')*(-1000:0);
      switch(DIVA_GUI_data.params.act_plot)
       case 1, % Motor Activity
        searchTarg=DIVA('MotorCortex','target',[],searchIdx);
        idx=getQueueIdx(searchTarg);
        motor=DIVA('VocalTract','motor',[],DIVA('TimeStep')*(-idx:0));
        targ = searchTarg(1000-idx+1:end);
        [targ motor]=chompIdx(targ,motor);

        if(~isempty(motor) && (mod(length(targ),DIVA('PlotStep'))==0 || ...
                              ~any(DIVA('busy'))))
          set(DIVA_GUI_data.private.model.guiaxes.other,'ylim',[0 1],...
                            'xlim',[min(targ) min(targ)+DIVA('AuditoryCortexCategorical','ntimepoints')]);
          plotReps(DIVA_GUI_data.private.model.traces.other,...
                   maxModeDim,DIVA('MotorCortex','nMotor'),...
                   targ,motor);
        end
        DIVA_GUI_data.params.act_plot = 1;

       case 2, % Auditory Signal
        searchTarg=DIVA('AuditoryCortex','target',[],searchIdx);
        idx=getQueueIdx(searchTarg);
        aurep=DIVA('AuditoryCortex','sound',[],DIVA('TimeStep')*(-idx:0));
        targ =searchTarg(1000-idx+1:end);
        [targ,aurep]=chompIdx(targ,aurep);

        if(~isempty(aurep) && (mod(length(targ),DIVA('PlotStep'))==0 || ...
                              ~any(DIVA('busy')))),
          set(DIVA_GUI_data.private.model.guiaxes.other,'ylim',[0 3500],...
                            'xlim',[min(targ) min(targ)+DIVA('AuditoryCortexCategorical','ntimepoints')]);
          plotReps(DIVA_GUI_data.private.model.traces.other,...
                   maxModeDim,DIVA('AuditoryCortex','nFormants'),...
                   targ,aurep);
        end
        DIVA_GUI_data.params.act_plot = 2;

       case 3, % Somatosensory Activity
        searchTarg=DIVA('SomatosensoryCortex','target',[],searchIdx);
        idx=getQueueIdx(searchTarg);
        somato=DIVA('SomatosensoryCortex','somatosensory',[],...
                    DIVA('TimeStep')*(-idx:0));
        targ  =searchTarg(1000-idx+1:end);
        [targ,somato]=chompIdx(targ,somato);

        if(~isempty(somato) && (mod(length(targ),DIVA('PlotStep'))==0 || ...
                               ~any(DIVA('busy'))))
          set(DIVA_GUI_data.private.model.guiaxes.other,'ylim',[0 1],...
                            'xlim',[min(targ) min(targ)+DIVA('AuditoryCortexCategorical','ntimepoints')]);
          plotReps(DIVA_GUI_data.private.model.traces.other,...
                   maxModeDim,DIVA('SomatosensoryCortex','nSensory'),...
                   targ,somato);
        end

       case 4, % Motor Weights
        searchTarg=DIVA('MotorCortex','target',[],searchIdx);
        idx=getQueueIdx(searchTarg);
        targ =searchTarg(1000-idx+1:end);

        motor = DIVA('MotorCortex','WeightsFromTargets');
        if(~isempty(motor) && DIVA('AuditoryCortexCategorical','match')~=-1 && ...
           (mod(length(targ),DIVA('PlotStep'))==0 || ~any(DIVA('busy'))))
          matchstart=DIVA('AuditoryCortexCategorical','match')*...
              DIVA('AuditoryCortexCategorical','ntimepoints')+1;
          matchend=matchstart+DIVA('AuditoryCortexCategorical','ntimepoints')-1;
          set(DIVA_GUI_data.private.model.guiaxes.other,'ylim',[0 1],...
                            'xlim',[matchstart matchend]);
          plotReps(DIVA_GUI_data.private.model.traces.other,...
                   maxModeDim,DIVA('MotorCortex','nMotor'),...
                   matchstart:matchend,motor(:,matchstart:matchend));
        end
        DIVA_GUI_data.params.act_plot = 4;

       case 5, % Somatosensory Weights
        searchTarg=DIVA('MotorCortex','target',[],searchIdx);
        idx=getQueueIdx(searchTarg);
        targ =searchTarg(1000-idx+1:end);
        somatorep=DIVA('SomatosensoryCortex','WeightsFromTargets');
        if(~isempty(somatorep) && DIVA('AuditoryCortexCategorical','match')~=-1 && ...
           (mod(length(targ),DIVA('PlotStep'))==0 || ~any(DIVA('busy'))))
          matchstart=DIVA('AuditoryCortexCategorical','match')*...
              DIVA('AuditoryCortexCategorical','ntimepoints')+1;
          matchend=matchstart+DIVA('AuditoryCortexCategorical','ntimepoints')-1;
          set(DIVA_GUI_data.private.model.guiaxes.other,'ylim',[0 1],...
                            'xlim',[matchstart matchend]);
          plotReps(DIVA_GUI_data.private.model.traces.other,...
                   maxModeDim,DIVA('SomatosensoryCortex','nSensory'),...
                   matchstart:matchend,somatorep(:,matchstart:matchend));
        end
        DIVA_GUI_data.params.act_plot = 5;

       case 6, % Acoustic Error
        searchTarg=DIVA('MotorCortex','target',[],searchIdx);
        idx=getQueueIdx(searchTarg);
        auerror=DIVA('MotorCortex','error_sound',[],DIVA('TimeStep')*(-idx:0));
        targ=searchTarg(1000-idx+1:end);
        [targ, auerror]=chompIdx(targ,auerror);
        if(~isempty(auerror) && (mod(length(targ),DIVA('PlotStep'))==0 || ...
                                ~any(DIVA('busy'))))
          set(DIVA_GUI_data.private.model.guiaxes.other,'ylim',[-750 750],...
                            'xlim',[min(targ) min(targ)+DIVA('AuditoryCortexCategorical','ntimepoints')]);
          plotReps(DIVA_GUI_data.private.model.traces.other,...
                   maxModeDim,DIVA('AuditoryCortex','nFormants'),...
                   targ,auerror);
        end
        DIVA_GUI_data.params.act_plot = 6;

       case 7, % Somatosensory Error Weights
        searchTarg=DIVA('MotorCortex','target',[],searchIdx);
        idx=getQueueIdx(searchTarg);
        somatoerror=DIVA('MotorCortex','error_somatosensory',[],...
                         DIVA('TimeStep')*(-idx:0));
        targ=searchTarg(1000-idx+1:end);
        [targ, somatoerror]=chompIdx(targ,somatoerror);

        if(~isempty(somatoerror) && (mod(length(targ),DIVA('PlotStep'))==0 || ...
                                    ~any(DIVA('busy'))))
          set(DIVA_GUI_data.private.model.guiaxes.other,'ylim',[-1 1],...
                            'xlim',[min(targ) min(targ)+DIVA('AuditoryCortexCategorical','ntimepoints')]);
          plotReps(DIVA_GUI_data.private.model.traces.other,...
                   maxModeDim,DIVA('SomatosensoryCortex','nSensory'),...
                   targ,somatoerror);
        end
        DIVA_GUI_data.params.act_plot = 7;
      end

      if(~any(DIVA('busy')) || mod(length(targ),DIVA('PlotStep'))==0),
        set(DIVA_GUI_data.private.model.guiaxes.other,...
            'ButtonDownFcn',...
            {@gui_plot_align, DIVA_GUI_data.private.figurehandle})
      end

      %--------------------------------------------------------------%

     case 'VT_Configuration',
      set(DIVA_GUI_data.private.vocaltract.guiaxes.vtplot,'visible','on');
      axes(DIVA_GUI_data.private.vocaltract.guiaxes.vtplot);
      if(ischar(varargin{indexargin+1}) && strcmp(varargin{indexargin+1},'update'))
        for i=1:length(DIVA_GUI_data.private.data.artics)
          config(i,1)=...
              get(DIVA_GUI_data.private.vocaltract.slider.(DIVA_GUI_data.private.data.artics{i}),...
                  'value');
        end
        for i=1:length(DIVA_GUI_data.private.data.glot_label)
          config(i+8,1)=...
              get(DIVA_GUI_data.private.vocaltract.slider.(DIVA_GUI_data.private.data.glot_label{i}),...
                  'value');
        end
        DIVA('VocalTract','plot',config(1:8,1));
      elseif(isstruct(varargin{indexargin+1})),
        if(~isempty(varargin{indexargin+1}.configuration))
          DIVA('VocalTract','plot',varargin{indexargin+1}.configuration(1:8,1));
        else
          DIVA('VocalTract','plot',zeros(DIVA('VocalTract','nArtic'),1));
        end
      else
        DIVA('VocalTract','plot',varargin{indexargin+1});
      end
      axis tight;

      %--------------------------------------------------------------%

     case 'artic_edit',
      params=varargin{indexargin+1};

      if(isstruct(params))
        for i=1:length(params)
          set(DIVA_GUI_data.private.vocaltract.edit.(params(i).name),...
              'string',sprintf('%1.2f',params(i).value));
        end
      else
        if(length(params)==length(DIVA_GUI_data.private.data.artics)),
          artics=DIVA_GUI_data.private.data.artics;
          for i=1:length(DIVA_GUI_data.private.data.artics),
            set(DIVA_GUI_data.private.vocaltract.edit.(artics{i}),...
                'string',sprintf('%1.2f',params(i)));
          end
        end
      end

      %--------------------------------------------------------------%

     case 'artic_slider',
      params=varargin{indexargin+1};
      if(~nargout)
        if(isstruct(params))
          for i=1:length(params)
            set(DIVA_GUI_data.private.vocaltract.slider.(params(i).name),...
                'value',params(i).value);
          end
        else
          if(length(params)==length(DIVA_GUI_data.private.data.artics)),
            artics=DIVA_GUI_data.private.data.artics;
            for i=1:length(DIVA_GUI_data.private.data.artics),
              set(DIVA_GUI_data.private.vocaltract.slider.(artics{i}),...
                  'value',params(i));
            end
          end
        end
      else
        if(isempty(params))
          params=DIVA_GUI_data.private.data.artics;
        end
        for i=1:length(params)
          out(i,1)=get(DIVA_GUI_data.private.vocaltract.slider.(params{i}),...
                       'value');
        end
      end

      %--------------------------------------------------------------%

     case 'glot_slider',
      params=varargin{indexargin+1};
      if(nargout)
        for i=1:length(params)
          out(i,1)=get(DIVA_GUI_data.private.vocaltract.slider.(params{i}),...
                       'value');
        end
      end

      %--------------------------------------------------------------%

     case 'auTarget'
      set(DIVA_GUI_data.private.model.label.target_file,'UserData',...
                        [varargin{indexargin+1}.pathname,...
                         varargin{indexargin+1}.filename]);
      set(DIVA_GUI_data.private.model.label.target_file,...
          'string',sprintf('%s',varargin{indexargin+1}.filename));
      DIVA('GUI','current_file',...
           [varargin{indexargin+1}.pathname,...
            varargin{indexargin+1}.filename]);
      set(DIVA_GUI_data.private.model.button.acquire,'enable','on');

      %--------------------------------------------------------------%

     case 'VTControl',
      if(isempty(varargin{indexargin+1}))
        aue=get(DIVA_GUI_data.private.guimenu.prefs.control.fmt,'checked');
        vte=get(DIVA_GUI_data.private.guimenu.prefs.control.artic,'checked');

        fnames=fieldnames(DIVA_GUI_data.private.auditory.edit);
        for i=1:length(fnames)
          set(DIVA_GUI_data.private.auditory.edit.(fnames{i}),'enable',aue);
        end
        set(DIVA_GUI_data.private.auditory.button.go,'enable',aue);

        fnames=fieldnames(DIVA_GUI_data.private.vocaltract.edit);
        for i=1:length(fnames)
          set(DIVA_GUI_data.private.vocaltract.edit.(fnames{i}),'enable',vte);
        end
        fnames=fieldnames(DIVA_GUI_data.private.vocaltract.slider);
        for i=1:length(fnames)
          if(strcmp(get(DIVA_GUI_data.private.vocaltract.slider.(fnames{i}),'tag'),'slider'))
            set(DIVA_GUI_data.private.vocaltract.slider.(fnames{i}),'enable',vte);
          end
        end
      end

      %--------------------------------------------------------------%
      
     case 'VTAction',
      if(strcmp(varargin{indexargin+1},'enable'))
        enablestring='on';
      else
        enablestring='off';
      end
      set(DIVA_GUI_data.private.vocaltract.button.generate,'enable',enablestring);
      set(DIVA_GUI_data.private.vocaltract.button.learn,'enable',enablestring);
      
      %--------------------------------------------------------------%

     case 'play',
      set(DIVA_GUI_data.private.model.button.play,'enable',varargin{indexargin+1});
      set(DIVA_GUI_data.private.guimenu.file.export.sound,...
          'enable',varargin{indexargin+1});
      set(DIVA_GUI_data.private.guimenu.file.export.movie,...
          'enable',varargin{indexargin+1});
      %--------------------------------------------------------------%

     case 'produce',
      set(DIVA_GUI_data.private.model.button.produce,...
          'enable',varargin{indexargin+1});

      %--------------------------------------------------------------%

     case 'save',
      SessionFolder=strcat(DIVA('SessionFolder'),filesep);
      if nargin<indexargin+1,
        initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
      else,
        initfile=[SessionFolder,'Session_',varargin{indexargin+1},...
                  filesep,mfilename,'.mat'];
      end
      params=DIVA_GUI_data.params;
      save(initfile,'params');

      %--------------------------------------------------------------%

     case 'disp',
      gfields=fieldnames(DIVA_GUI_data.params);
      for i=1:length(gfields)
        disp(['---',gfields{i}]);
        disp(DIVA_GUI_data.params.(gfields{i}));
      end

      %--------------------------------------------------------------%

     case 'exit',
      if ishandle(DIVA_GUI_data.private.figurehandle),
        set(DIVA_GUI_data.private.figurehandle,'deletefcn','');
        delete(DIVA_GUI_data.private.figurehandle);
      end
      clear DIVA_GUI_data;
      disp([mfilename, ' : Exiting session...']);

      % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      % CODE IMPLEMENTING THIS MODULE PROCESS(ES) %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      % Note: use DIVA(ModuleName,ChannelName,data,delay) to comunicate with other
      %       DIVA modules
      % case 'ChannelName1',
      % case 'ChannelName2',

      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     otherwise,
      if isfield(DIVA_GUI_data.params,varargin{indexargin}),
        if indexargin==nargin,
          out=DIVA_GUI_data.params.(varargin{indexargin});
        else,
          DIVA_GUI_data.params.(varargin{indexargin})=varargin{indexargin+1};
        end
      else,
        warning('DIVA_GUI: wrong argument');
      end
    end
  end

function plotReps(handles,maxModeDim,curDim,targ,rep)
  lo={'-','--'};
  co=get(get(handles(1),'parent'),'colororder');
  for n=1:maxModeDim,
    if(n<=curDim),
      set(handles(n),'Xdata',targ,'Ydata',rep(n,:),'visible','on',...
                     'buttondownfcn',@gui_plot_align);
      if(curDim>DIVA('AuditoryCortex','nFormants') && ...
         n<=DIVA('MotorCortex','nMotor')),
        set(handles(n),'linestyle',lo{mod(n,2)+1});
        if(mod(n,2)==0)
          set(handles(n),'color',co(mod(n-1,11)+1,:));
        else
          set(handles(n),'color',co(mod(n,11)+1,:));
        end
      else,
        set(handles(n),'linestyle',lo{1},'color',co(mod(n,11)+1,:));
      end
    else
      set(handles(n),'visible','off');
    end
  end
