function out=DIVA_MotorCortex(varargin)
% DIVA_MotorCortex Motor cortex model
%
% This model receives the forward and feedback motor commands and
% sends control commands to the vocal tract
%
% DIVA_MotorCortex('init' [,SessionName]);          Initializes the module
% DIVA_MotorCortex('save' [,SessionName] );         Saves state
% DIVA_MotorCortex('exit');                         Exits the module (without saving)
% DIVA_MotorCortex(PropertyName [,PropertyValue] )  Reads and writes internal
%                                                   model properties
%
% DIVA_MotorCortex('target',idx,'error_sound',e1,'error_somatosensory',e2);
% The motor target is represented as an index 'idx' into the a set
% of learned weights ('WeightsFromTargets') representing the
% learned forward commands. The auditory and somatosensory errors
% are represented as vectors of AuditoryCortex and
% SomatosensoryCortex activations respectively.  Corrective
% movements are calculated from the auditory and somatosensory
% error via explicit pseudoinverse of the ForwardModel.
%
% Current DIVA_MotorCortex properties are: (* requires re-initialization)
%     eps_motor                     : integration constant for motor commands
%     eps_decay                     : decay constant for motor commands
%                                     (tendency towards zero)
%     eps_learn                     : integration constant for feedforward learning
%  *  delayToVocalTract             : delay (in seconds) for signals to VocalTract
%     WeightFeedForward             : weight of feedforward command to total motor command
%     WeightFeedBack                : weight of feedback command to total motor command
%     WeightLearn                   : weight of feedback command to learning of
%                                     total motor command
%     WeightFeedBack_sound          : weight of auditory feedback to total feedback command
%     WeightFeedBack_somatosensory  : weight of somatosensory feedback to total
%                                     feedback command
%     WeightLearn_sound             : weight of auditory feedback to total learning command
%     WeightLearn_somatosensory     : weight of somatosensory feedback to total
%                                     learning command
%     WeightsFromTargets            : weights for target signals
%     ErrorNoise_sound		    : additive noise which simulates degradation of transmission of auditory error cell information
%     ErrorNoise_somatosensory      : additive noise which simulates degradation of transmission of somatosensory error cell information
%     ErrorNoise_ffAmplitude	    : additive noise which simulates degradation of transmission of the feedforward command
%

% Dependencies:
%

% 2007-03-22: (JSB) Additive normal noise distortion added to .current.error_sound 
% and .current.error_somatosensory channels simulating degraded synaptic transmission
% between calculation of auditory/somatosensory feedback corrective motor
% commands and combination with complete motor command.  Done in motor space
% to use standard (motor) units.  Also, in this space a single noise
% parameter makes sense (can split later on for individual articulators)

% TODO % Adaptive control of feedforward/feedback learning weights
  
  out=[];
  global DIVA_MotorCortex_data

  %%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%
  %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

  somethingtoexecute=0;
  for indexargin=1:2:nargin,
    switch(varargin{indexargin}),
     case 'init',
      SessionFolder=strcat(DIVA('SessionFolder'),filesep);
      if nargin<indexargin+1 || isempty(varargin{indexargin+1}),
        initfile='';
      else,
        initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
      end
      if isempty(initfile) || isempty(dir(initfile)),
        disp([mfilename, ' : Defining new session...']);
        DIVA_MotorCortex_data.params = struct(...
            'nMotor',2*DIVA('VocalTract','nArtic'),...  % # of neurons for motor representation
            'eps_motor',.25,...          % integration constant for motor commands
            'eps_decay',.95,...          % decay constant for motor commands (tendency towards zero)
            'delayToVocalTract',.005,...
            'WeightFeedForward',0.9,...  % Weight of feedforward command to total motor command 
            'WeightFeedBack',0.1,...     % Weight of feedback command to total motor command 
            'WeightFeedBack_sound',0.8,... % Weight of auditory feedback to feedback command (used for total motor command)
            'WeightFeedBack_somatosensory',.0,... % Weight of somatosensory feedback to feedback command (used for total motor command)
            'WeightLearn',0.7,...          % Weight of feedback command to learning of motor command
            'WeightLearn_sound',0.6,...    % Weight of auditory feedback to feedback command (used for learning)
            'WeightLearn_somatosensory',0,... % Weight of somatosensory feedback to feedback command (used for learning) 
            'ErrorNoise_sound', [0 0], ... % mean, std of additive noise to auditory error
            'ErrorNoise_somatosensory', [0 0], ... % mean, std of additive noise to somatosensory error
            'ErrorNoise_ffAmplitude', [0 0], ... % mean, std of additive noise to feedforward command amplitude
            'ErrorNoise_ffTiming', [0,0], ... % mean and std of additive noise to feedforward command timint
            'WeightsFromTargets',[],...    % Motor command weights
            'InverseParameters',struct(... % Parameters needed for explicit pseudoinverse
                'DELTA', 5,...
                'ALPHA', 0.5,...
                'BETA', 0.05,...
                'LAMBDA',0),...
            'plotf',0);
      else,
        data=load(initfile,'-mat');
        DIVA_MotorCortex_data.params=data.params;
      end

      DIVA_MotorCortex_data.params.TimeStep=DIVA('TimeStep');
      DIVA_MotorCortex_data.params.default_motorSound = ...
          (0.5*ones(DIVA_MotorCortex_data.params.nMotor,1));
      % Default motor representation when defining a new target

      DIVA_MotorCortex_data.params.default_motorSilence = ...
          (0.5*ones(DIVA_MotorCortex_data.params.nMotor,1));
      % Motor representation associated with relaxed vocal tract configuration

      DIVA_MotorCortex_data.params.motorAmplitude = DIVA('VocalTract','Amplitude');
      % Motor representation index for amplitude control

      DIVA_MotorCortex_data.params.MaedaSpread = DIVA('VocalTract','MaedaSpread');
      DIVA_MotorCortex_data.params.delayFromSomatosensoryCortex=...
          DIVA('SomatosensoryCortex','delayToMotorCortex');
      DIVA_MotorCortex_data.params.delayFromAuditoryCortex=...
          DIVA('AuditoryCortex','delayToMotorCortex');
      DIVA_MotorCortex_data.params.delayToerror_sound=...
          DIVA_MotorCortex_data.params.delayToVocalTract+...
          DIVA('VocalTract','delayToSoundChannel')+...
          0*DIVA('SoundChannel','delayToCochlea')+...
          DIVA('Cochlea','delayToAuditoryCortex')+...
          DIVA('AuditoryCortex','delayToMotorCortex');
      DIVA_MotorCortex_data.params.delayToerror_somatosensory=...
          DIVA_MotorCortex_data.params.delayToVocalTract+...
          DIVA('VocalTract','delayToSensoryChannel')+...
          0*DIVA('SensoryChannel','delayToSensory')+...
          DIVA('Sensory','delayToSomatosensoryCortex')+...
          DIVA('SomatosensoryCortex','delayToMotorCortex');
      DIVA_MotorCortex_data.params.current.motor=...
          DIVA_MotorCortex_data.params.default_motorSilence;
      DIVA_MotorCortex_data.params.current.error_sound=[];
      DIVA_MotorCortex_data.params.current.error_somatosensory=[];
      DIVA_MotorCortex_data.params.current.target=[];
      % temporary variables used to calculate true WeightsFromTargets
      DIVA_MotorCortex_data.params.current.motorupdate_a=0;
      DIVA_MotorCortex_data.params.current.motorupdate_s=0;

      % Channels
      out={'error_sound','error_somatosensory','target'};

     case 'save',
      SessionFolder=strcat(DIVA('SessionFolder'),filesep);
      if nargin<indexargin+1,
        initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
      else,
        initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
      end
      params=DIVA_MotorCortex_data.params;
      save(initfile,'params');

     case 'exit',
      clear DIVA_MotorCortex_data

     case 'disp',
      disp(DIVA_MotorCortex_data.params);
      out=fieldnames(DIVA_MotorCortex_data.params);

      %%%%%%%%%%%%%%%% ACTION %%%%%%%%%%%%%%%%%
      %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

     case 'error_sound'
      % current sound error projection
      
      DIVA_MotorCortex_data.params.current.error_sound = varargin{indexargin+1};
      somethingtoexecute=1;
      if ~nargout,
        DIVA('GUI','other_weights','error_sound');
        DIVA('ModelStatePlot','MotorCortex','error_sound');
      end

     case 'error_somatosensory'
      % current somatosensory error projection
      DIVA_MotorCortex_data.params.current.error_somatosensory = varargin{indexargin+1};
      somethingtoexecute=1;
      if ~nargout,
        DIVA('GUI','other_weights','error_somatosensory');
        DIVA('ModelStatePlot','MotorCortex','error_somatosensory');
      end

     case 'target'
      % current target projection
      DIVA_MotorCortex_data.params.current.target=varargin{indexargin+1};
      somethingtoexecute=1;
      if ~nargout,
        DIVA('ModelStatePlot','MotorCortex','target');
      end

     otherwise,
      if isfield(DIVA_MotorCortex_data.params,varargin{indexargin}),
        if indexargin==nargin,
          out=DIVA_MotorCortex_data.params.(varargin{indexargin});
        else,
          if(strmatch('errornoise',lower(varargin{indexargin}))),
            if(length(varargin{indexargin+1})==1)
              DIVA_MotorCortex_data.params.(varargin{indexargin}) = ...
                [0 varargin{indexargin+1}];
            else
              DIVA_MotorCortex_data.params.(varargin{indexargin}) = ...
                varargin{indexargin+1};
            end
          else
            DIVA_MotorCortex_data.params.(varargin{indexargin})=varargin{indexargin+1};
          end
        end
      else,
        warning('DIVA_MotorCortex: wrong argument');
      end
    end
  end

  %---------------------------EXECUTE---------------------------------------%


  if somethingtoexecute,
    ntimepoints=DIVA('AuditoryCortexCategorical','ntimepoints');
    if(DIVA_MotorCortex_data.params.WeightLearn > 0), % Skip if simply production
      %----------------------COMPUTE FEEDBACK COMPONENTS----------------------%
      if ~isempty(DIVA_MotorCortex_data.params.current.error_sound) && ...
            DIVA_MotorCortex_data.params.WeightFeedBack_sound>0,
        %-----------------------Auditory Error Feedback-----------------------%
        % PERFORM Auditory to Motor Transformation
        % explicit pseudo-inverse implementation
        originalmotorcommand=DIVA('VocalTract','motor',[],...
                                  -DIVA_MotorCortex_data.params.delayToerror_sound+...
                                  DIVA_MotorCortex_data.params.delayToVocalTract+...
                                  DIVA_MotorCortex_data.params.TimeStep*...
                                  (0:size(DIVA_MotorCortex_data.params.current.error_sound,2)-1));

        originaltarget=DIVA('AuditoryCortex','target',[],...
                            -DIVA_MotorCortex_data.params.delayFromAuditoryCortex+...
                            DIVA_MotorCortex_data.params.TimeStep*...
                            (0:size(DIVA_MotorCortex_data.params.current.error_sound,2)-1));

        originalsound=DIVA('AuditoryCortex','sound',[],...
                           -DIVA_MotorCortex_data.params.delayFromAuditoryCortex+...
                           DIVA_MotorCortex_data.params.TimeStep*...
                           (0:size(DIVA_MotorCortex_data.params.current.error_sound,2)-1));

        errorsound    = DIVA_MotorCortex_data.params.current.error_sound;
        nMotor        = DIVA_MotorCortex_data.params.nMotor;
        nArtic        = DIVA('VocalTract','nArtic');
        nFormants     = DIVA('AuditoryCortex','nFormants');

        N=size(DIVA_MotorCortex_data.params.current.error_sound,2); %length of error sound

        tmp=DIVA('VocalTract','motor',originalmotorcommand);
        artic=tmp{2}(1:nArtic,:);
        
        % Compute Auditory Error to Motor Corrective Movement
        motor = calculateInverse(originalmotorcommand,originalsound,errorsound,nFormants);
        % Additive error representing degraded synaptic transmission
        motor = motor + (DIVA_MotorCortex_data.params.ErrorNoise_sound(1) + ...
                         DIVA_MotorCortex_data.params.ErrorNoise_sound(2) * ...
                         randn(size(motor)));
        DIVA_MotorCortex_data.params.current.error_sound = motor;
        CommandFeedBack_sound=DIVA_MotorCortex_data.params.current.error_sound;
      else
        CommandFeedBack_sound = 0;
      end

      %---------------------Somatosensory Error Feedback---------------------%

      % PERFORM Somatosensory to Motor Transformation
      if ~isempty(DIVA_MotorCortex_data.params.current.error_somatosensory) && ...
            DIVA_MotorCortex_data.params.WeightFeedBack_somatosensory > 0,
        % explicit pseudo-inverse implementation
        originalmotorcommand = ...
            DIVA('VocalTract','motor',[],...
                 -DIVA_MotorCortex_data.params.delayToerror_somatosensory+...
                 DIVA_MotorCortex_data.params.delayToVocalTract+...
                 DIVA_MotorCortex_data.params.TimeStep*...
                 (0:size(DIVA_MotorCortex_data.params.current.error_somatosensory,2)-1));

        originaltarget = ...
            DIVA('SomatosensoryCortex','target',[],...
                 -DIVA_MotorCortex_data.params.delayFromSomatosensoryCortex+...
                 DIVA_MotorCortex_data.params.TimeStep*...
                 (0:size(DIVA_MotorCortex_data.params.current.error_somatosensory,2)-1));

        originalsomato = ...
            DIVA('SomatosensoryCortex','somatosensory',[],...
                 -DIVA_MotorCortex_data.params.delayFromSomatosensoryCortex+...
                 DIVA_MotorCortex_data.params.TimeStep*...
                 (0:size(DIVA_MotorCortex_data.params.current.error_somatosensory,2)-1));

        nArtic        = DIVA('VocalTract','nArtic');
        nMotor        = DIVA_MotorCortex_data.params.nMotor;
        nSomato       = DIVA('Sensory','nSensory');
        errorsomato   = DIVA_MotorCortex_data.params.current.error_somatosensory;

        N = size(DIVA_MotorCortex_data.params.current.error_somatosensory,2);

        % Compute Somatosensory Error to Motor Corrective Movement
        motor = calculateInverse(originalmotorcommand,originalsomato,errorsomato,nSomato);
        % Additive error representing degraded synaptic transmission
        motor = motor + (DIVA_MotorCortex_data.params.ErrorNoise_somatosensory(1) + ...
                         DIVA_MotorCortex_data.params.ErrorNoise_somatosensory(2) * ...
                         randn(size(motor)));
        
        DIVA_MotorCortex_data.params.current.error_somatosensory=motor;
        CommandFeedBack_somatosensory=DIVA_MotorCortex_data.params.current.error_somatosensory;
      else
        CommandFeedBack_somatosensory = 0;
      end

      %-------------------------Combined Error Feedback--------------------------%

      % Combine Auditory and Somatosensory corrective motor commands
      CommandFeedBack = ...
          (DIVA_MotorCortex_data.params.WeightFeedBack_sound * CommandFeedBack_sound) + ...
          (DIVA_MotorCortex_data.params.WeightFeedBack_somatosensory * ...
           CommandFeedBack_somatosensory);

    else, % There is no learning going on
      CommandFeedBack=0;
    end

    %-------------------------Prepare Motor Movement w.r.t. Motor targets------%

    if ~isempty(DIVA_MotorCortex_data.params.current.target),
      % initializes target if it does not exist
      idx=union(floor(DIVA_MotorCortex_data.params.current.target),...
                ceil(DIVA_MotorCortex_data.params.current.target));

      % Make room at end of WeightsFromTargets if current utterance is not represented yet
      if max(idx)+ntimepoints>size(DIVA_MotorCortex_data.params.WeightsFromTargets,2),
        idxnew=size(DIVA_MotorCortex_data.params.WeightsFromTargets,2)+...
               1:max(idx)+ntimepoints;

        DIVA_MotorCortex_data.params.WeightsFromTargets(:,idxnew) = ...
            DIVA_MotorCortex_data.params.default_motorSound(:,ones(1,length(idxnew)));
      end

      % Determine current Motor Target region
      % HERE IS WHERE WE ADD temporal NOISE for FEEDFORWARD CMD
      %   - Represents demylinization
      if length(idx)>1,
        DIVA_MotorCortex_data.params.current.targetmotor = ...
            fastinterp1(idx,DIVA_MotorCortex_data.params.WeightsFromTargets(:,idx),...
                        DIVA_MotorCortex_data.params.current.target);
      else,
        DIVA_MotorCortex_data.params.current.targetmotor = ...
            DIVA_MotorCortex_data.params.WeightsFromTargets(:,idx*ones(1,size(DIVA_MotorCortex_data.params.current.target,2)));
      end
      if nargout>0,
        tempcurrentmotor = DIVA_MotorCortex_data.params.current.motor;
        DIVA_MotorCortex_data.params.current.motor = ...
            DIVA_MotorCortex_data.params.default_motorSilence;
      end

      %-------------------------Feedforward AND total Command-------------------------%

      for n1=1:size(DIVA_MotorCortex_data.params.current.target,2),
        % computes feedforward component
        CommandFeedForward = ...
            DIVA_MotorCortex_data.params.current.targetmotor(:,n1) - ...
            DIVA_MotorCortex_data.params.current.motor;
        % Additive noise simulating degradation in synaptic transmission of feedforward command
        CommandFeedForward = CommandFeedForward + ...
                             (DIVA_MotorCortex_data.params.ErrorNoise_ffAmplitude(1) + ...
                              DIVA_MotorCortex_data.params.ErrorNoise_ffAmplitude(2) * ...
                              randn(size(CommandFeedForward)));          

        % computes motor command from FeedForward and FeedBack commands
        Command = DIVA_MotorCortex_data.params.WeightFeedBack*...
                  CommandFeedBack(:,min(n1,size(CommandFeedBack,2))) + ...
                  DIVA_MotorCortex_data.params.WeightFeedForward*CommandFeedForward;

        % Calculate current motor command hard-limited to [0,1]
        DIVA_MotorCortex_data.params.current.motor = ...
            max(0,min(1,DIVA_MotorCortex_data.params.default_motorSilence + ...
                      DIVA_MotorCortex_data.params.eps_decay*...
                      (DIVA_MotorCortex_data.params.current.motor - ...
                       DIVA_MotorCortex_data.params.default_motorSilence) + ...
                      DIVA_MotorCortex_data.params.eps_motor * Command));
        out(:,n1)=(DIVA_MotorCortex_data.params.current.motor);
      end

      %---------------------------Project to Vocal Tract---------------------------%

      if ~nargout,
        DIVA('VocalTract','motor',out,DIVA_MotorCortex_data.params.delayToVocalTract);
      else,
        DIVA_MotorCortex_data.params.current.motor=tempcurrentmotor;
      end
    end

    %-----------------------------------------------------------------------------%

    % Update Learned Weights

    if DIVA_MotorCortex_data.params.WeightLearn > 0,
      % learning from auditory error
      if ~isempty(DIVA_MotorCortex_data.params.current.error_sound) && ...
            DIVA_MotorCortex_data.params.WeightLearn_sound > 0,

        % Find relevant target indicies
        targettolearn = DIVA('MotorCortex','target',[],...
                             -DIVA_MotorCortex_data.params.delayToerror_sound + ...
                             DIVA_MotorCortex_data.params.TimeStep*...
                             (0:size(CommandFeedBack_sound,2)));

        % Sometimes in blockmode the tail target index is nonzero, errant behavior with
        % no known source yet, fix to zero...2006-10-06, JB
        if(targettolearn(end)<targettolearn(end-1)),
          targettolearn(end)=0;
        end

        if ~all(targettolearn(1:end-1)>0),
          disp('DIVA_MotorCortex: warning, receiving sound error signal without a motor cause');
        else,
          idxvalid = 1:length(targettolearn)-1;
          idx = floor(targettolearn(idxvalid(1))):floor(targettolearn(idxvalid(end)));
          idx = idx(find(idx>0));

          % Temporary weight update, need to find average update over all instances
          % of target indicies per training trial
          % For instance, target index sequence might be [1 1.3 1.6 1.9 2.0,...]
          % The round() command above creates a sequence [1 1 2 2 2,...], resulting in
          % over training of index 2 compared to index 1.  The imaginary part of
          % motorupdate_* counts the number of consecutive input indicies for use in an
          % averaging procedure
          if(~DIVA('BlockMode')),
            DIVA_MotorCortex_data.params.current.motorupdate_a=...
                DIVA_MotorCortex_data.params.current.motorupdate_a+...
                DIVA_MotorCortex_data.params.WeightLearn * ...
                DIVA_MotorCortex_data.params.WeightLearn_sound * ...
                CommandFeedBack_sound + sqrt(-1);

            % Update Motor Weights
            if diff(floor(targettolearn)),
              %idx = union(floor(targettolearn),ceil(targettolearn));
              idx = min(floor(targettolearn(find(targettolearn>0)))):max(ceil(targettolearn(find(targettolearn>0))));
              idx = idx(find(idx>0));
              if(diff(floor(targettolearn))<0)
                idx = idx;%max(round(targettolearn))
              else
                idx = idx(find(idx<floor(targettolearn(end))));
              end
              DIVA_MotorCortex_data.params.WeightsFromTargets(:,idx)=...
                  DIVA_MotorCortex_data.params.WeightsFromTargets(:,idx)+...
                  repmat(real(DIVA_MotorCortex_data.params.current.motorupdate_a)./...
                         imag(DIVA_MotorCortex_data.params.current.motorupdate_a),...
                         1,length(idx));
              DIVA_MotorCortex_data.params.current.motorupdate_a=0;
            end
          else
            idx=unique(floor(targettolearn(find(targettolearn>0))));
            blockBound = diff(floor(targettolearn));
            for n=1:length(idx),
              if(blockBound(max(find(floor(targettolearn)==idx(n))))>0)
                targetIdx = ...
                    idx(n):idx(n)+blockBound(max(find(floor(targettolearn)==idx(n))))-1;
              else
                targetIdx = idx(n):idx(n)+1;
              end
              DIVA_MotorCortex_data.params.WeightsFromTargets(:,targetIdx) = ...
                  DIVA_MotorCortex_data.params.WeightsFromTargets(:,targetIdx) + ...
                  DIVA_MotorCortex_data.params.WeightLearn * ...
                  DIVA_MotorCortex_data.params.WeightLearn_sound * ...
                  repmat(...
                      sum(CommandFeedBack_sound(:,find(floor(targettolearn)==idx(n))),2) / ...
                      length(find(floor(targettolearn)==idx(n))),...
                      1,length(targetIdx));
            end
          end
        end
      end

      %----------------------------------------------------------------------------%

      % learning from somatosensory error
      if ~isempty(DIVA_MotorCortex_data.params.current.error_somatosensory) & ...
            DIVA_MotorCortex_data.params.WeightLearn_somatosensory,
        % Find relevant target indicies
        targettolearn = ...
            DIVA('MotorCortex','target',[],...
                 -DIVA_MotorCortex_data.params.delayToerror_somatosensory + ...
                 DIVA_MotorCortex_data.params.TimeStep*...
                 (0:size(CommandFeedBack_somatosensory,2)));
        
        % Sometimes in blockmode the tail target index is nonzero, errant behavior with
        % no known source yet, fix to zero...2006-10-06, JB
        if(targettolearn(end)<targettolearn(end-1)),
          targettolearn(end)=0;
        end
        
        if ~all(targettolearn(1:end-1)>0),
          disp(['DIVA_MotorCortex: warning, receiving somatosensory error '...
                'signal without a motor cause']);
        else,
          idxvalid=1:length(targettolearn)-1;
          idx = floor(targettolearn(idxvalid(1))):floor(targettolearn(idxvalid(end)));
          idx = idx(find(idx>0));

          % See explanation for motorupdate_* above
          if(~DIVA('BlockMode')),
            DIVA_MotorCortex_data.params.current.motorupdate_s=...
                DIVA_MotorCortex_data.params.current.motorupdate_s+...
                DIVA_MotorCortex_data.params.WeightLearn * ...
                DIVA_MotorCortex_data.params.WeightLearn_somatosensory * ...
                CommandFeedBack_somatosensory + sqrt(-1);

            % Update Motor Weights
            if(diff(floor(targettolearn))),
              idx = min(floor(targettolearn(find(targettolearn>0)))):max(ceil(targettolearn(find(targettolearn>0))));
              idx = idx(find(idx>0));
              if(diff(floor(targettolearn))<0)
                idx = idx;
              else
                idx = idx(find(idx<floor(targettolearn(end))));
              end
              DIVA_MotorCortex_data.params.WeightsFromTargets(:,idx)=...
                  DIVA_MotorCortex_data.params.WeightsFromTargets(:,idx)+...
                  repmat(real(DIVA_MotorCortex_data.params.current.motorupdate_s)./...
                         imag(DIVA_MotorCortex_data.params.current.motorupdate_s),...
                         1,length(idx));
              DIVA_MotorCortex_data.params.current.motorupdate_s=0;
            end
          else
            idx=unique(floor(targettolearn(find(targettolearn>0))));
            blockBound = diff(floor(targettolearn));
            for n=1:length(idx),
              if(blockBound(max(find(floor(targettolearn)==idx(n))))>0)                  
                targetIdx = ...
                    idx(n):idx(n)+blockBound(max(find(floor(targettolearn)==idx(n))))-1;
              else
                targetIdx = idx(n):idx(n)+1;
              end
              DIVA_MotorCortex_data.params.WeightsFromTargets(:,targetIdx) = ...
                  DIVA_MotorCortex_data.params.WeightsFromTargets(:,targetIdx) + ...
                  DIVA_MotorCortex_data.params.WeightLearn * ...
                  DIVA_MotorCortex_data.params.WeightLearn_somatosensory * ...
                  repmat(...
                      sum(CommandFeedBack_somatosensory(:,find(floor(targettolearn)==idx(n))),2) / ...
                      length(find(floor(targettolearn)==idx(n))),...
                      1,length(targetIdx));
            end
          end
        end
      end
    end
    DIVA_MotorCortex_data.params.current.error_sound=[];
    DIVA_MotorCortex_data.params.current.error_somatosensory=[];
    DIVA_MotorCortex_data.params.current.target=[];

  end

