function out=DIVA_Sensory(action,varargin);
% DIVA_Sensory: DIVA Sensory model
%
% DIVA_Sensory('init' [,SessionName]);          Initializes the module
% DIVA_Sensory('save' [,SessionName] );         Saves state 
% DIVA_Sensory('exit');                         Exits the module (without saving)
% DIVA_Sensory(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%
% DIVA_Sensory('configuration',s,delay);  
% Sends the vocal tract configuration in vector s (see output of VocalTract module) 
% to the sensory model with a given delay. The module computes the proprioceptive
% and somatosensory consequences associated with the configuration s, and
% sends them to the SomatosensoryCortex module.
%
% Current DIVA_Sensory properties are: (* requires re-initialization)
%   *  delayToSomatosensoryCortex  : delay (in seconds) for signals to SomatosensoryCortex
%

% Dependencies: 
%

out=[];
global DIVA_Sensory_data
if nargin<1, 
  action='init'; 
end

switch(lower(action)),     
  %%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%
  %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%
 case 'init',
  SessionFolder=strcat(DIVA('SessionFolder'),filesep);
  if nargin<2 | isempty(varargin{1}), 
    initfile=''; 
  else, 
    initfile=[SessionFolder,'Session_',varargin{1},filesep,mfilename,'.mat']; 
  end
  if isempty(initfile) | isempty(dir(initfile)),
    disp([mfilename, ' : Defining new session...']);
    DIVA_Sensory_data.params=struct(...
        'MaedaSpread',DIVA('VocalTract','MaedaSpread'),... % VocalTract boundary values
        'nMotor',DIVA('VocalTract','nArtic'),... % Dimension of Motor representation
        'delayToSomatosensoryCortex',.005);
    % Transformations for proprioceptive sensory representation
    DIVA_Sensory_data.params.Motor2Proprioceptive=...
        cat(2,.5*ones(2*DIVA_Sensory_data.params.nMotor,1), ...
            kron(eye(DIVA_Sensory_data.params.nMotor),...
                 [.5;-.5]/DIVA_Sensory_data.params.MaedaSpread));
    % Transformation for tactile sensory representation
    DIVA_Sensory_data.params.Maeda2Palate = ...
        diag(1./[9,8,7,5,4,9])*...
        [4.975 0.391 -1.528  0.372 1.710  0; ...
         3.802 0.394 -1.577  0.616 1.449  0; ...
         2.138 0.194 -1.444  0.638 0.793  0; ...
         0.378 0.283 -1.229  0.626 0.322  0; ...
         -0.484 0.388 -0.891  0.703 0.391  0; ...
         4.919 0.271  0.028 -0.004 -0.001 -1.8114];
    
    % # of neurons for sensory representation
    DIVA_Sensory_data.params.nSensory = ...
        size(DIVA_Sensory_data.params.Motor2Proprioceptive,1)+...
        size(DIVA_Sensory_data.params.Maeda2Palate,1); 
    
  else,
    data=load(initfile,'-mat');
    DIVA_Sensory_data.params=data.params;
  end
  out={};
  
 case 'save',
  SessionFolder=strcat(DIVA('SessionFolder'),filesep);
  if nargin<2, 
    initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat']; 
  else, 
    initfile=[SessionFolder,'Session_',varargin{1},filesep,mfilename,'.mat']; 
  end
  params=DIVA_Sensory_data.params;
  save(initfile,'params'); 
  
 case 'exit',
  clear DIVA_Sensory_data;
  
 case 'disp',
  disp(DIVA_Sensory_data.params);
  out=fieldnames(DIVA_Sensory_data.params);
  
  %%%%%%%%%%%%%%%% ACTION %%%%%%%%%%%%%%%%%
  %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

 case 'configuration',
  Ar = varargin{1}(1:DIVA_Sensory_data.params.nMotor,:); 
  Maeda = varargin{1}(DIVA_Sensory_data.params.nMotor+1:end,:); 
  if nargin<3 | isempty(varargin{2}), 
    delay=0; 
  else, 
    delay=varargin{2}; 
  end
  % Calculate Sensory representation of current motor
  % configuration, with a hard-limit [0,1]
  out = cat(1,DIVA_Sensory_data.params.Motor2Proprioceptive*[ones(1,size(Ar,2));Ar],... 
            max(0, min(1, DIVA_Sensory_data.params.Maeda2Palate*[ones(1,size(Maeda,2));...
                      Maeda(1:5,:)])));
  
  % projects output to SomatosensoryCortex
  if ~nargout,
    DIVA('SomatosensoryCortex','somatosensory',out,...
         delay+DIVA_Sensory_data.params.delayToSomatosensoryCortex);
  end
  
 otherwise,
  if isfield(DIVA_Sensory_data.params,action),
    if nargin<2, 
      out=DIVA_Sensory_data.params.(action); 
    else, 
      DIVA_Sensory_data.params.(action)=varargin{1}; 
    end
  else, 
    warning('DIVA_Sensory: wrong argument'); 
  end
end
