function out=DIVA_SomatosensoryCortex(varargin);
% DIVA_SomatosensoryCortex Somatosensory cortex model
%
% This model computes the cortical representation of a somatosensory input and 
% its difference with the somatosensory target.
%
% DIVA_SomatosensoryCortex('init' [,SessionName]);          Initializes the module
% DIVA_SomatosensoryCortex('save' [,SessionName] );         Saves state
% DIVA_SomatosensoryCortex('exit');                         Exits the module (without saving)
% DIVA_SomatosensoryCortex(PropertyName [,PropertyValue] )  Reads and writes internal model
%                                                           properties
%
% DIVA_SomatosensoryCortex('target',s2,'somatosensory',s1);
% The model computes the difference between the target signal s2 and the signal s1 and sends it
% to motor cortex (DIVA_MotorCortex). The somatosensory targets are slowly adapted (learning) to
% predict the associated somatosensory inputs.
%
% The signal s1 is a Sensory representation (see DIVA_Sensory) and the signal s2
% is an indexed categorical representation (see DIVA_SoundMap). The model stores
% weights, WeightsFromTargets used for the comparison between
% targets and current Sensory representation
%
% Current DIVA_Somatosensory properties are: (* requires re-initialization)
%  *   delayToMotorCortex          : delay (in seconds) for signals to MotorCortex
%      eps_learn                   : integration constant for somatosensory learning
%      WeightsFromTargets          : Weights for target signals (somatosensory targets)

% Dependencies:
%

out=[];
global DIVA_SomatosensoryCortex_data

%%%%%%%%%%%%%%%% INITIALIZATION %%%%%%%%%
%%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

for indexargin=1:2:nargin,
  switch(varargin{indexargin}),
   case 'init',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1 | isempty(varargin{indexargin+1}),
      initfile='';
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end
    if isempty(initfile) | isempty(dir(initfile)),
      disp([mfilename, ' : Defining new session...']);
      DIVA_SomatosensoryCortex_data.params=struct(...
          'eps_learn',.3,...             % integration constant for somatosensory learning
          'delayToMotorCortex',.005,...
          'WeightsFromTargets',[]);      % Weights for target signals (somatosensory targets)

    else,
      data=load(initfile,'-mat');
      DIVA_SomatosensoryCortex_data.params=data.params;
    end
    DIVA_SomatosensoryCortex_data.params.nSensory = DIVA('Sensory','nSensory');
    DIVA_SomatosensoryCortex_data.params.TimeStep = DIVA('TimeStep');
    DIVA_SomatosensoryCortex_data.params.default_somatosensory = ...
        zeros(DIVA_SomatosensoryCortex_data.params.nSensory,1);
    DIVA_SomatosensoryCortex_data.params.current.target = [];

    % Parameter used to appropriately update the
    % WeightsFromTargets, see below
    DIVA_SomatosensoryCortex_data.params.current.somatoupdate = 0;
    out={'target','somatosensory'};

   case 'save',
    SessionFolder=strcat(DIVA('SessionFolder'),filesep);
    if nargin<indexargin+1,
      initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
    else,
      initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
    end
    params=DIVA_SomatosensoryCortex_data.params;
    save(initfile,'params');

   case 'exit',
    clear DIVA_SomatosensoryCortex_data;

   case 'disp',
    disp(DIVA_SomatosensoryCortex_data.params);
    out=fieldnames(DIVA_SomatosensoryCortex_data.params);

    %%%%%%%%%%%%%%%% ACTION %%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%% %%%%%%%%%%%%%%%%%%%%%%%%

   case 'target'
    % current somatosensory target projection
    DIVA_SomatosensoryCortex_data.params.current.target=varargin{indexargin+1};
    if ~nargout,
      DIVA('ModelStatePlot','SomatosensoryCortex','target');
    end

   case 'somatosensory'
    ntimepoints = DIVA('AuditoryCortexCategorical','ntimepoints');
    DIVA_SomatosensoryCortex_data.params.current.somatosensory = varargin{indexargin+1};
    if ~isempty(DIVA_SomatosensoryCortex_data.params.current.target),
      % initializes target if it does not exist

      % Need 'round' incoming target indices due to time-normalization
      % in AuditoryCortexCategorical
      idx = union(floor(DIVA_SomatosensoryCortex_data.params.current.target),...
                  ceil(DIVA_SomatosensoryCortex_data.params.current.target));

      % Add enough space to end of WeightsFromTargets for "new" target representations
      if max(idx)+ntimepoints>size(DIVA_SomatosensoryCortex_data.params.WeightsFromTargets,2),
        idxnew=size(DIVA_SomatosensoryCortex_data.params.WeightsFromTargets,2)+1:max(idx)+ntimepoints;
        DIVA_SomatosensoryCortex_data.params.WeightsFromTargets(:,idxnew) = ...
            DIVA_SomatosensoryCortex_data.params.default_somatosensory(:,ones(1,length(idxnew)));
      end

      % Get Current Target
      if length(idx)>1,
        DIVA_SomatosensoryCortex_data.params.current.targetsomatosensory = ...
            fastinterp1(idx,DIVA_SomatosensoryCortex_data.params.WeightsFromTargets(:,idx),...
                        DIVA_SomatosensoryCortex_data.params.current.target);
      else,
        DIVA_SomatosensoryCortex_data.params.current.targetsomatosensory = ...
            DIVA_SomatosensoryCortex_data.params.WeightsFromTargets(:,idx*ones(1,size(DIVA_SomatosensoryCortex_data.params.current.target,2)));
      end

      % Error computation, simple difference between targets and current configuration
      out = DIVA_SomatosensoryCortex_data.params.current.targetsomatosensory - ...
            DIVA_SomatosensoryCortex_data.params.current.somatosensory;

      if ~nargout,
        % Projects output to MotorCortex
        DIVA_SomatosensoryCortex_data.params.current.error=out;
        DIVA('MotorCortex',...
             'error_somatosensory',...
             DIVA_SomatosensoryCortex_data.params.current.error,...
             DIVA_SomatosensoryCortex_data.params.delayToMotorCortex);

        % Learning new Somatosensory Weights
        if DIVA_SomatosensoryCortex_data.params.eps_learn > 0,
          % Find Weights associated with input
          targettolearn = DIVA('SomatosensoryCortex','target',[],...
                               DIVA_SomatosensoryCortex_data.params.TimeStep*...
                               (0:size(DIVA_SomatosensoryCortex_data.params.current.target,2)));
          if(targettolearn(end)<targettolearn(end-1))
            targettolearn(end)=0;
          end
          if ~all(targettolearn(1:end-1)>0),
            disp('DIVA_SomatosensoryCortex: warning, receiving empty target');
          else,
            idxvalid = 1:length(targettolearn)-1;
            idx=floor(targettolearn(idxvalid(1))):floor(targettolearn(idxvalid(end)));
            idx = idx(find(idx>0));
            
            DIVA_SomatosensoryCortex_data.temp.targettolearn=targettolearn;
            DIVA_SomatosensoryCortex_data.temp.parser=diff(floor(targettolearn));
            
            % Temporary weight update, need to find average update over all instances
            % of target indicies per training trial
            % For instance, target index sequence might be [1 1.3 1.6 1.9 2.0,...]
            % The round() command above creates a sequence [1 1 2 2 2,...], resulting in
            % over training of index 2 compared to index 1.  The imaginary part of
            % somatoupdate counts the number of consecutive input indicies for use in an
            % averaging procedure
            if(~DIVA('BlockMode'))
              DIVA_SomatosensoryCortex_data.params.current.somatoupdate=...
                  DIVA_SomatosensoryCortex_data.params.current.somatoupdate * ...
                  (1-DIVA_SomatosensoryCortex_data.params.eps_learn) + ...
                  DIVA_SomatosensoryCortex_data.params.current.somatosensory * ...
                  DIVA_SomatosensoryCortex_data.params.eps_learn + sqrt(-1);
            
              if(diff(floor(targettolearn))),
                idx = min(floor(targettolearn(find(targettolearn>0)))):max(ceil(targettolearn(find(targettolearn>0))));
                idx = idx(find(idx>0));
                if(diff(floor(targettolearn))<0)
                  idx = idx;
                else
                  idx = idx(find(idx<floor(targettolearn(end))));
                end
                
                DIVA_SomatosensoryCortex_data.params.WeightsFromTargets(:,idx)=...
                    DIVA_SomatosensoryCortex_data.params.WeightsFromTargets(:,idx)+...
                    repmat(real(DIVA_SomatosensoryCortex_data.params.current.somatoupdate)./...
                           imag(DIVA_SomatosensoryCortex_data.params.current.somatoupdate),...
                           1,length(idx));
                DIVA_SomatosensoryCortex_data.params.current.somatoupdate=...
                    zeros(DIVA_SomatosensoryCortex_data.params.nSensory,1);
              end
            else
              idx=unique(floor(targettolearn(find(targettolearn>0))));
              blockBound = diff(floor(targettolearn));
              for n=1:length(idx),
                if(blockBound(max(find(floor(targettolearn)==idx(n))))>0)                  
                  targetIdx = ...
                      idx(n):idx(n)+blockBound(max(find(floor(targettolearn)==idx(n))))-1;
                else, %end of block
                  targetIdx = idx(n):idx(n)+1;
                end
                DIVA_SomatosensoryCortex_data.params.WeightsFromTargets(:,targetIdx) = ...
                    DIVA_SomatosensoryCortex_data.params.WeightsFromTargets(:,targetIdx) * ...
                    (1-DIVA_SomatosensoryCortex_data.params.eps_learn) + ...
                    DIVA_SomatosensoryCortex_data.params.eps_learn * ...
                    repmat(...
                        sum(DIVA_SomatosensoryCortex_data.params.current.somatosensory(:,find(floor(targettolearn)==idx(n))),2) / ...
                        length(find(floor(targettolearn)==idx(n))),...
                        1,length(targetIdx));
              end
            end
          end
        end
      end
      DIVA_SomatosensoryCortex_data.params.current.target=[];
    end
    if ~nargout, % try to plot model state information
      DIVA('ModelStatePlot','SomatosensoryCortex','somatosensory');
      DIVA('GUI','other_weights','somatosensory');
      DIVA('GUI','other_weights','somatosensory_weight');
    end

   otherwise,
    if isfield(DIVA_SomatosensoryCortex_data.params,varargin{indexargin}),
      if indexargin==nargin,
        out=DIVA_SomatosensoryCortex_data.params.(varargin{indexargin});
      else,
        DIVA_SomatosensoryCortex_data.params.(varargin{indexargin})=varargin{indexargin+1};
      end
    else,
      warning('DIVA_SomatosensoryCortex: wrong argument');
    end
  end
end

