function out=DIVA_VocalTract(varargin)
% DIVA_VocalTract DIVA Vocal tract module
%
% Computes the vocal tract parameters from any given motor
% configuration command, and its acoustic consequences.
%
% DIVA_VocalTract('init' [,SessionName]);          Initializes the module
% DIVA_VocalTract('save' [,SessionName] );         Saves state
% DIVA_VocalTract('exit');                         Exits the module (without saving)
% DIVA_VocalTract(PropertyName [,PropertyValue] )  Reads and writes internal model properties
%
% DIVA_VocalTract('motor',s)   Computes the Maeda vocal tract configuration parameters
% associated with a motor configuration 's', and sends this and the motor command to the
% Sensory module. It also computes the sound associated with this configuration and
% sends this to the Cochlear module.
%
% Current DIVA_VocalTract properties are:
%   delayToSoundChannel       : delay (in seconds) for signals to SoundChannel module
%   delayToSensoryChannel     : delay (in seconds) for signals to SensoryChannel module

% note: Relationship between motor articulatory representation and Maeda
% vocal tract representation
% Ar(1,:)      : Jaw aperture          Maeda(1,:)
% Ar(2:4,:)    : Tongue shape          Maeda(2:4,:)
% Ar(5,:)      : Lip protrusion        Maeda(6,:)
% Ar(6,:)      : Larynx height         Maeda(7,:)
% Ar(7:8,:)    : Lip aperture          Maeda(5,:)
% Ar(9,:)      : Glotal opening        Ag0, Maeda(8,:)
% Ar(10,:)     : Glotal pressure       AgP, Maeda(9,:)
% Ar(11,:)     : F0                    F0, Maeda(10,:)

% Dependencies: @d_opvt (Maeda synthesizer)
%


  out=[];
  global DIVA_VocalTract_data

  for indexargin=1:2:nargin,
    switch(varargin{indexargin}),
     case 'init',
      SessionFolder=strcat(DIVA('SessionFolder'),filesep);
      if nargin<indexargin+1 | isempty(varargin{indexargin+1}),
        initfile='';
      else,
        initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
      end
      if isempty(initfile) | isempty(dir(initfile)),
        disp([mfilename, ' : Defining new session...']);
        % INITIALIZES INTERNAL MODULE PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        DIVA_VocalTract_data.params=struct(...
            'nMaeda',7,...                    % # of components of Maeda representation;
            'nArtic',8,...                    % # of components in Articulatory rep.
            'MaedaSpread',4,...               % Spread of Maeda parameters
            'JawLip',0.2,...
            'Amplitude',10,...                % index to amplitude control
            'usesynth',0,...                  % use the Maeda synthesizer
            'vtcalc_fs',10000,...             % internal sampling frequency used by Maeda
            'fs',10000,...                    % sampling frequency of synthesized sound
            'noise',[],...                    % noise sample
            'delayToSoundChannel',.020,...
            'delayToSensoryChannel',.005);

        % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      else,
        data=load(initfile,'-mat');
        DIVA_VocalTract_data.params=data.params;
      end
      % INITIALIZES OTHER (TEMPORAL) PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      DIVA_VocalTract_data.params.vt = d_opvt;
      DIVA_VocalTract_data.params.TimeStep = DIVA('TimeStep');
      DIVA_VocalTract_data.params.delayToForwardModel=...
          DIVA_VocalTract_data.params.delayToSoundChannel+0*...
          DIVA('SoundChannel','delayToCochlea')+DIVA('Cochlea','delayToForwardModel');


      DIVA_VocalTract_data.params.current=struct(...
          'sound',[],...
          'configuration',[]);
      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

      % LIST OF MODULE CHANNELS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      out={'motor'};
      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     case 'save',
      SessionFolder=strcat(DIVA('SessionFolder'),filesep);
      if nargin<indexargin+1,
        initfile=[SessionFolder,'Session_','default',filesep,mfilename,'.mat'];
      else,
        initfile=[SessionFolder,'Session_',varargin{indexargin+1},filesep,mfilename,'.mat'];
      end
      params=DIVA_VocalTract_data.params;
      save(initfile,'params');

     case 'exit',
      clear DIVA_VocalTract_data;
      % CODE FOR CLEARING MEMORY OR THE LIKE %%%%%%%%%%%%%%%%%%%%%%
      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     case 'disp',
      disp(DIVA_VocalTract_data.params);
      out=fieldnames(DIVA_VocalTract_data.params);

      % CODE IMPLEMENTING THIS MODULE PROCESS(ES) %%%%%%%%%%%%%%%%%

     case 'motor', % Assume Articulatory dimension input (8xn matrix)
      Ar0=varargin{indexargin+1};
      if(isempty(Ar0)),
        Ar0 = zeros(DIVA_VocalTract_data.params.nArtic,1);
      end
      
      % If input is not articulatory assume its Motor == (2*8 x n matrix)
      % and convert to Articulatory configuration
      if(size(Ar0,1)>DIVA_VocalTract_data.params.nArtic), 
        Ar0=M2Ar(Ar0);
      end
      Ar=Ar0;

      % Computes Maeda parameters
      Maeda=zeros(DIVA_VocalTract_data.params.nMaeda,size(Ar,2));
      Maeda([1:4,6:7],:) = Ar(1:6,:);
      Art_val = -1.2;
      Ar_old = Ar;
      max_pos = 2;
      min_pos = 0.5;

      % lip aperture parameters
      Ar(7,:) = (Ar(7,:)+DIVA_VocalTract_data.params.MaedaSpread)/...
                (2*DIVA_VocalTract_data.params.MaedaSpread)*(max_pos-(-min_pos))+(-min_pos);
      Ar(8,:) = (Ar(8,:)+DIVA_VocalTract_data.params.MaedaSpread)/...
                (2*DIVA_VocalTract_data.params.MaedaSpread)*(min_pos-(-max_pos))+(-max_pos);

      % lip revisited
      val  = (Ar(7,:)<Ar(8,:)).*Ar(8,:);
      dAr7 = (val~=0).*((val-(-min_pos))/(max_pos-(-min_pos))*...
                        (2*DIVA_VocalTract_data.params.MaedaSpread)-...
                        DIVA_VocalTract_data.params.MaedaSpread-Ar_old(7,:));
      Ar_new = Ar_old;
      Ar_new(7,:) = Ar_old(7,:)+dAr7;
      Ar(7,:) = (Ar_new(7,:)+DIVA_VocalTract_data.params.MaedaSpread)/...
                (2*DIVA_VocalTract_data.params.MaedaSpread)*(max_pos-(-min_pos))+(-min_pos);
      Maeda(5,:) = Art_val+(DIVA_VocalTract_data.params.MaedaSpread-Art_val)*...
          max(Ar(7,:)-Ar(8,:),0)/(2*max_pos);

      % Hart-limit maeda parameters to +-MaedaSpread
      Maeda = min(max(Maeda,-DIVA_VocalTract_data.params.MaedaSpread),...
                  DIVA_VocalTract_data.params.MaedaSpread);

      % hard-limit articulatory parameters
      Ar(1:8,:) = min(max(Ar(1:8,:),-DIVA_VocalTract_data.params.MaedaSpread),...
                      DIVA_VocalTract_data.params.MaedaSpread);

      % Computes sound, currently using only the forward model (23/08/2006)
      if DIVA_VocalTract_data.params.usesynth>0,
        % use Maeda synthesizer
        if size(Maeda,2)==1,
          data = ...
              synthX(DIVA_VocalTract_data.params.vt, ...
                     Maeda(:,ones(1,DIVA_VocalTract_data.params.windowsizeOverTimeStep)),...
                     DIVA_VocalTract_data.params);
        else,
          data = synthX(DIVA_VocalTract_data.params.vt, Maeda, ...
                        DIVA_VocalTract_data.params);
        end

        if isempty(DIVA_VocalTract_data.params.noise),
          data.sig = data.sig;
        else,
          data.sig = ...
              data.sig+...
              DIVA_VocalTract_data.params.noise(1+mod(0:length(data.sig)-1,...
                                                      length(DIVA_VocalTract_data.params.noise)));
        end
        out=data.sig;

      else, % Determine auditory output via ForwardModel
        out = DIVA('ForwardModel','motor',Maeda(1:7,:));
      end

      DIVA_VocalTract_data.params.current.sound = out;
      DIVA_VocalTract_data.params.current.configuration = [Ar0;Maeda];

      if ~nargout,
        % Send data across channels
        DIVA('SoundChannel','sound',out,DIVA_VocalTract_data.params.fs,...
             DIVA_VocalTract_data.params.delayToSoundChannel);
        DIVA('SensoryChannel','configuration',[Ar0;Maeda],...
             DIVA_VocalTract_data.params.delayToSensoryChannel);
        DIVA('ForwardModel','motor',Ar0,...
             DIVA_VocalTract_data.params.delayToForwardModel);
        % Plotting commands
        DIVA('GUI','other_weights','motor');
        DIVA('GUI','other_weights','motor_weight');
        DIVA('ModelStatePlot','VocalTract','motor');
      else,
        out={DIVA_VocalTract_data.params.current.sound, ...
             DIVA_VocalTract_data.params.current.configuration};
      end

     case 'synthsound',
      % use the Maeda synthesizer to output sound based on motor trajectories
      % if argument is empty, synthesize the last produced motor trajectory, if possible

      if(isempty(varargin{indexargin+1})),
        % find indices via GUI plotting functionality
        % velocity scale via AuditoryCortexCategorical target output

      else, % synthesize the trajectory supplied, can be either
            % articulatory or motor dimensionality
        
        if(size(varargin{indexargin+1},1)==DIVA_VocalTract_data.params.nArtic+3), %artic
          outm = DIVA('VocalTract','motor',...
                      varargin{indexargin+1}(1:DIVA_VocalTract_data.params.nArtic,:));
          outg = varargin{indexargin+1}(DIVA_VocalTract_data.params.nArtic+1:end,:);
        else, %motor
          nMotor = DIVA('MotorCortex','nMotor');
          outm = DIVA('VocalTract','motor',varargin{indexargin+1}(1:nMotor,:));
          outg = varargin{indexargin+1}(nMotor+1:end,:);
        end
          
        data.fs = DIVA_VocalTract_data.params.vtcalc_fs;
        data.duration = size(varargin{indexargin+1},2);
        
        % For Maeda Synthesizer
        data.X = outm{2}(DIVA_VocalTract_data.params.nArtic+1:end,:);
        
        % For Alfonso Synthesizer
        %  data.X = (outm{2}(1:DIVA_VocalTract_data.params.nMaeda,:) + ...
        %            DIVA_VocalTract_data.params.MaedaSpread) ./ ...
        %           (2*DIVA_VocalTract_data.params.MaedaSpread);
        data.Ag0 = [[0:data.duration-1]',...
                    outg(1,:)',...
                    ones(data.duration,1)];
        data.AgP = [[0:data.duration-1]',...
                    outg(2,:)',...
                    ones(data.duration,1)];
        data.F0  = [[0:data.duration-1]',...
                    outg(3,:)',...
                    ones(data.duration,1)];
      end
      % For Alfonso Synthesizer
      % sound = synthesizer([data.X;outg],0.001);
      % out = {sound.sig,sound.fs};
      % For Maeda Synthesizer
      ssound = synth1(DIVA_VocalTract_data.params.vt,data);
      out = {0.99*ssound./(max(abs(ssound(:)))),DIVA_VocalTract_data.params.fs};

     case 'plot', %assume Articulatory input (n=[8;3])
      % Plot the "Divaman" vocaltract

      out=DIVA('VocalTract','motor',varargin{indexargin+1});
      Maeda=out{2}(DIVA_VocalTract_data.params.nArtic+1:end,:);
      Ar=out{2}(1:DIVA_VocalTract_data.params.nArtic,:);
      plot(DIVA_VocalTract_data.params.vt,Maeda,...
           sum(Ar(1:8,:),1)~=0,Ar);

      % %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

     otherwise,
      if isfield(DIVA_VocalTract_data.params,varargin{indexargin}),
        if indexargin==nargin,
          out=DIVA_VocalTract_data.params.(varargin{indexargin});
        else,
          DIVA_VocalTract_data.params.(varargin{indexargin})=varargin{indexargin+1};
        end
      else,
        warning('DIVA_VocalTract: wrong argument');
      end
    end
  end

