function [y_fit,params]=ppf(x,y,control,order,continuity,weight,type);
% PPF Piece-wise Polynomial Fit
%
% Y_FIT=PPF(X,Y,X0,N,M);
% fits Y=f(X) to a N-th order polynomial fit with X0 knots or control points. 
% X is a vector and Y can be a vector or a matrix (colums are observations),
% X0 can be a scalar (number of control points) or a vector (control points),
% N is the order of the polynomial fit, and M is the order of continuity imposed 
% (one for continuity of first-derivatives, two for continuity in the second-derivatives, etc.).
% N defaults to 3, M defaults to N-1, and X0 defaults to X
%
% [Y_FIT,PARAMS]=PPF(...) also returns the parameters of the fit
% Y_NEW=PPF(X_NEW,PARAMS) estimates new data using previous fit
%
% % example 1: Fitting sequential points in a curve
% x=2:9; y=rand(size(x)); 
% [y_fit,params]=ppf(x,y); 
% x_new=linspace(1,10,1000);
% y_new=ppf(x_new,params); 
% plot(x,y,'bo',x_new,y_new,'r-'); set(gca,'xtick',params.control); grid on;
%
% % example 2: Approximating data points to a polynomial curve
% x=sort(rand(1,200)); y=x.*(sin(6*pi*x)+.5*randn(size(x))); 
% [y_fit,params]=ppf(x,y,10); % 10 control points
% plot(x,y,'bo',[x;x],[y;y_fit],'b:',x,y_fit,'r.-'); set(gca,'xtick',params.control); grid on;
%

% alfnie@bu.edu
% 05/06

x=x(:)'; sX=size(x);
if isstruct(y),
    params=y;
    ncontrol=length(params.control);
    if ~params.type,
        x=ppf_normdata(sX(2),params.control,ncontrol,x);
        X=ppf_createX(sX(2),0:ncontrol-1,params.order,params.continuity,params.weight,ncontrol,x);
    else,
        X=ppf_createX(sX(2),params.control,params.order,params.continuity,params.weight,ncontrol,x);
    end
    y_fit=params.b'*X'; 
else,
    if sum(size(y)>1)<=1, y=y(:)'; end
    if nargin<3 || isempty(control), control=unique(x); end
    if nargin<4 || isempty(order), order=3; end
    if nargin<5 || isempty(continuity), continuity=order-1; end
    if nargin<6 || isempty(weight), weight=.01; end % when underdetermined, (weight<1) weights more highest order terms
    if nargin<7 || isempty(type), type=1; end % (type=0) equalizes x data prior to polynomial fitting 
    sY=size(y);
    
    if prod(size(control))==1, 
        sx=sort(x);
        ncontrol=control;
        control=sx(round(linspace(1,sX(2),ncontrol)));
    else, ncontrol=length(control); end
    if ~type,
        x=ppf_normdata(sX(2),control,ncontrol,x);
        X=ppf_createX(sX(2),0:ncontrol-1,order,continuity,weight,ncontrol,x);
    else,
        X=ppf_createX(sX(2),control,order,continuity,weight,ncontrol,x);
    end
    b=pinv(1e-10*eye(size(X,2))+X'*X)*X'*y';
    y_fit=b'*X'; 
    if nargout>1,
        params.order=order;
        params.continuity=continuity;
        params.control=control;
        params.b=b;
        params.weight=weight;
        params.type=type;
        %params.X=X;
    end
end

function X=ppf_createX(sX,control,order,continuity,weight,ncontrol,x);
X=ones(sX,order+(order-continuity)*(ncontrol-1)); for n2=1:order-1, X(:,1+n2)=(x').^n2; end; 
for n2=1:order-continuity, 
    t1=(max(0,x'-control(1)).^(eps+order-n2+1)); 
    for n1=1:ncontrol-1, 
        t2=(max(0,x'-control(n1+1)).^(eps+order-n2+1)); 
        X(:,order+(order-continuity)*(n1-1)+n2)=(weight)*(t1-t2); 
        t1=t2;
    end
end

function x=ppf_normdata(sX,control,ncontrol,x)
r=sum(repmat(control',[1,sX])<repmat(x,[ncontrol,1]),1);
idx=find(~r); x(idx)=0; 
idx=find(r==ncontrol); x(idx)=ncontrol-1; 
idx=find(r>0 & r<ncontrol); x(idx)=r(idx)-1+(x(idx)-control(r(idx)))./(control(r(idx)+1)-control(r(idx)));
