
function [Y_fit,params]=ppr(X,Y,varargin);
% PPR Projection-Pursuit non-linear Regression
%
% Y_FIT=PPR(X,Y) fits a PPR model to the data Y_FIT = Sum_i{ f_i(w_i'*X) }
% minimizing the sum of error squares sum(sum(abs(Y_FIT-Y).^2));
% X and Y are a predictor and output matrices, respectively (columns are observations)
% The functions f_i are estimated using piece-wise polynomial fitting (see PPF for additional options)
%
% optional arguments:
%   'components'    : number of components extracted (default size(X,1))
%   'maxiter'       : number of iterations (default 50*size(X,1))
%   'disp'          : display on/off (defaults 1=on)
%   'control'       : PPF number of control points (default 5)
%   'order'         : PPF order of polynomial fit (default 3)
%   'continuity'    : PPF order of continuity (default 2)
%
% [Y_FIT,PARAMS]=PPR(...) also returns the parameters of the fit
% Y_NEW=PPR(X_NEW,PARAMS) estimates new data using the previous fit
%
% example:
% X=randn(6,1e3); 
% n=3; m=5; 
% Y=.01*randn(1,size(X,2)); for n1=1:m, idx=ceil(size(X,1)*rand(ceil(rand*min(n,size(X,1))),1)); Y=Y+(prod(X(idx,:))); end
% [Y_fit,params]=ppr(X,Y);
%



% alfnie@bu.edu
% 05/06

fields={'components','size(X,1)',...    % Number of components
        'maxiter','50*size(X,1)',...    % Number of iterations for the estimation of each component
        'decay','((params.maxiter:-1:1)/params.maxiter)/3',... % S.D. of random steps during estimation of each component
        'tol',.01,...                   % When reaching R2=1-tol the algorithm stops
        'resort',.90,...                % If resort*R2 of a new component is larger than R2 of the previous component it will resort them
        'disp',1,...                    % Graphic display on/off
        'control','5',...               % PPF parameter
        'order',[],...                  % PPF parameter
        'continuity',[],...             % PPF parameter
        'weight',[],...                 % PPF parameter
        'type',[]};                     % PPF parameter
params=[]; for n1=1:2:nargin-2, params=setfield(params,lower(varargin{n1}),varargin{n1+1}); end
for n1=1:2:length(fields), if ~isfield(params,fields{n1}) | isempty(getfield(params,fields{n1})), if isstr(fields{n1+1}), params=setfield(params,fields{n1},eval(fields{n1+1})); else, params=setfield(params,fields{n1},fields{n1+1}); end; end; end

if ~isstruct(Y), % fit
    sX=size(X);
    params.comp0=mean(Y,2);
    Y=Y-repmat(params.comp0,[1,size(Y,2)]);
    e0=sum(sum(abs(Y).^2));

    params.comp=[];
    Y0=Y;
    rnow=1;
    ncomp=1;
    while ncomp<=params.components & rnow>params.tol,
        if params.disp, cumdisp; end
        if length(params.comp)>=ncomp, % if already computed this component
            w=params.comp(ncomp).w;
            [Y_fit,Pfit]=ppf(w'*X,params.comp(ncomp).ppf);
            err=sum(sum(abs(Y_fit-Y).^2));
            if params.disp, subplot(2,params.components,params.components+ncomp); plot(w'*X,Y,'b.',w'*X,Y_fit,'r.'); set(gca,'xtick',[]); end
            if params.disp, cumdisp(['C',num2str(ncomp),'=',num2str(100*(rnow-err/e0),'%04.1f'),'%% ']); end
        else,
            % projection pursuit
            w=zeros(sX(1),1); 
            err=inf;
            ERR=nan+zeros(1,params.maxiter);
            for niter=1:params.maxiter, 
                w_new=w+params.decay(niter)*randn(sX(1),1); w_new=w_new/(eps+sqrt(sum(abs(w_new).^2)));
                [Y_fit_new,Pfit_new]=ppf(w_new'*X,Y,params.control,params.order,params.continuity,params.weight,params.type);
                err_new=sum(sum(abs(Y_fit_new-Y).^2));
                if err_new<err, 
                    err=err_new; w=w_new; Y_fit=Y_fit_new; Pfit=Pfit_new;
                    if params.disp>1, idxplot=ceil(rand*size(Y,1)); subplot(2,params.components,params.components+ncomp); plot(w'*X,Y(idxplot,:),'b.',w'*X,Y_fit(idxplot,:),'r.'); set(gca,'color','y','xtick',[]); end
                    if params.disp, cumdisp(['C',num2str(ncomp),'=',num2str(100*(rnow-err/e0),'%04.1f'),'%% ']); end
                end
                ERR(niter)=err_new/e0;
                if params.disp>1, subplot(2,params.components,ncomp); plot(ERR(1:niter),'.-'); set(gca,'color','y'); drawnow; end
            end
            if params.disp, subplot(2,params.components,ncomp); plot(ERR(1:params.maxiter),'.-'); if ncomp==1, ylabel('error'); end; set(gca,'ylim',[0,1]); box off; end
        end
        params.comp(ncomp).ppf=Pfit;
        params.comp(ncomp).w=w;
        params.comp(ncomp).r=rnow-err/e0;
        if params.disp, idxplot=ceil(rand*size(Y,1)); subplot(2,params.components,params.components+ncomp); plot(w'*X,Y(idxplot,:),'b.',w'*X,Y_fit(idxplot,:),'r.'); if ncomp==1, h=ylabel('Y'); set(h,'rotation',0); end; h=xlabel(['z_{',num2str(ncomp),'}']); set(gca,'xtick',[]); h=title(['C',num2str(ncomp),' = ',num2str(100*params.comp(ncomp).r,'%2.1f'),'%']); set(h,'units','norm','position',[.4,.9,0],'fontweight','bold'); set(gcf,'color','w','doublebuffer','on'); drawnow; end;
        rnow=err/e0;
        Y=Y-Y_fit;
        if params.resort && ncomp>1,
            rall=cat(1,params.comp(1:ncomp).r);
            idx=find((.01+rall(1:end-1))./rall(end)<params.resort);
            if ~isempty(idx),
                params.comp=params.comp([1:idx(1)-1,ncomp]);
                Y=Y0;
                rnow=1;
                ncomp=0;
                if params.disp, fprintf('\n'); end
            end
        end
        ncomp=ncomp+1;
    end
    params.components=ncomp-1;
    Y_fit=repmat(params.comp0,[1,size(Y,2)])+Y0-Y;
else,   % estimate
    params=Y;
    Y_fit=repmat(params.comp0,[1,size(X,2)]);
    for ncomp=1:params.components,
        Y_fit=Y_fit+ppf(params.comp(ncomp).w'*X,params.comp(ncomp).ppf);
    end
end

