function data = synthesizer(X,dt,idxremove)
% SYNTHESIZER    Synthesizes the output from the vocal tract
% data = synthesizer(X,dt);
% Where X is a 10xN matrix containing N time-points of Maeda parameters
%       X(1,:)      : Jaw aperture  (0 to 1; 1=jaw high)
%       X(2:4,:)    : Tongue shape  (0 to 1; 1/1/1=tongue back/tongue high/tongue tip high)
%       X(5,:)      : Lip aperture  (0 to 1; 1=lips open)
%       X(6,:)      : Lip protrusion  (0 to 1; 1=lip protruded)
%       X(7,:)      : Larynx height (0 to 1; 1=larynx high)
%       X(8,:)      : Voicing (0 to 1; 1=voiced)
%       X(9,:)      : Pressure (0 to 1)
%       X(10,:)     : F0 (in Hz)
% and data is a structure with fields:
%       sig         : sound wave
%       Fs          : sampling frequency
%       Af          : area function
%       Plot        : vocal tract contour
%

% alfnie@bu.edu


persistent doam

if nargin<3, 
  idxremove=[];
end

X0=cat(2,X(:,1),X);
X(:,idxremove)=[];
X=cat(2,X(:,1),X);
Kmaeda=4;
[N,Nt]=size(X);
if N~=10,
  error('wrong dimensionality');
end

if isempty(doam),
  load('synthesizer.mat');
end
if 0, % use this to initialize Area function and VocalTract contour fitting functions
    X=max(-Kmaeda,min(Kmaeda, 2*randn(7,1e4)));
    Af=zeros([2,17,size(X,2)]); P1=zeros([4,29,size(X,2)]);
    for n1=1:size(X,2),
      [nill,nill,nill,Af(:,:,n1),nill,P1(:,:,n1)]=doAM(X(:,n1));
    end
    cd('fcn');
    if 1,
      Y=cat(1,shiftdim(Af(1,:,:),1),shiftdim(20*Af(2,1,:),1));
      [Y_fit,params]=ppr(cat(1,X,ones(1,size(X,2))),Y,...
                         'components',32,'control',5,'order',2);
        disp(100*(1-sum(sum(abs(Y-Y_fit).^2))/sum(sum(abs(Y-repmat(mean(Y,2),[1,size(Y,2)])).^2))));
        load synthesizer.mat;
        doam.Af=params;
        save synthesizer.mat doam;
    end
    if 1,
        Y=reshape(P1,4*29,size(P1,3));
        [Y_fit,params]=ppr(cat(1,X,ones(1,size(X,2))),Y,...
                           'components',16,'control',5,'order',2);
        disp(100*(1-sum(sum(abs(Y-Y_fit).^2))/sum(sum(abs(Y-repmat(mean(Y,2),[1,size(Y,2)])).^2))));
        load synthesizer.mat;
        doam.VocalTract=params;
        save synthesizer.mat doam;
    end
    cd('..');
end

% Computes Area function and Vocaltract contour
t=ppr(cat(1,Kmaeda*(2*X(1:7,:)-1),ones(1,Nt)),doam.Af);
data.Af=t(1:end-1,:);
data.dx=repmat(t(end,:)/20,[size(t,1)-1,1]);
t=ppr(cat(1,Kmaeda*(2*X(1:7,:)-1),ones(1,Nt)),doam.VocalTract);
opt.P1=reshape(t,[4,29,size(t,2)]);
opt.Af=data.Af;

Fs=(size(data.Af,1))*1000; 
data.fs=Fs;
Np=round(dt*(Nt-1)*Fs);

% Modifies area function (physical models)
Af=data.Af/50;
xAf=linspace(0,1,size(data.Af,1))';
Af=Af.*repmat(.5+1.5*sin(.9*pi*xAf),[1,size(data.Af,2)]);
% interaction of articulator-pressure and air-pressure
pAf=.001*(X(9+zeros(1,size(Af,1)),:).^2)+.5*Af;
Af=max( pAf, Af );
idx=find(diff(min(Af,[],1)>0,1,2)==1);
Af=max(0, Af);
Af=2*max(0, Af).^1.5;
Af=cat(1,.01+zeros(1,size(Af,2)),Af,ones(1,size(Af,2)));
Nl=31;
for n1=1:length(idx),
  % pressure build-up
  X(9,min(Nt,1+idx(n1)+(0:Nl-1))) = ...
      1*(1+10*linspace(1,0,Nl).^2.*max(0,-.5+X(9,min(Nt,1+idx(n1)+(0:Nl-1))))).*...
      X(9,min(Nt,1+idx(n1)+(0:Nl-1)));
end;

% Generates impulse response
rf=lpcaa2rf(max(eps,Af'));
[ar,arp,aru,g]=lpcrf2ar(rf);  %g=abs(g).^1.5;
im=lpcar2im(ar./(eps+g(:,ones(1,size(ar,2)))),3*64)';
im=im.*repmat(exp(-linspace(0,5,size(im,1)))',[1,size(im,2)]);

% Generates source
sfilt=hamming(round(Fs*dt)+1); sfilt=sfilt/sum(sfilt);
F0=repmat(60+60*X0(10,:),[round(Fs*dt),1]); F0=convn(F0(:),sfilt,'valid');
F0(repmat(round(Fs*dt)*(idxremove(:)-1)',[round(Fs*dt),1])+repmat((1:round(Fs*dt))',[1,length(idxremove)]))=[]; % correct phase when synthesizing only a portion of a sound

Ag0=repmat(X(8,:),[round(Fs*dt),1]); 
Ag0=convn(Ag0(:),sfilt,'valid');

AgP=repmat(X(9,:),[round(Fs*dt),1]); 
AgP=convn(AgP(:),sfilt,'valid');

Ag0=max(.01,min(1,1-Ag0)).^2;
phase=cumsum(F0/Fs);
%phase=cumsum((F0+10*randn(size(F0)))/Fs);
noise=convn(randn([Np,1]),convn(exp(-.0005*(0:100).^2)',[-1;.5;1]),'same'); 
noise=noise-mean(noise); noise=noise/sqrt(mean(abs(noise).^2))/4;

glottal=glotlf(2,phase,[.8,.085,.2])+10*glotlf(1,phase,[.8,.085,.2]); 
glottal=convn(glottal-mean(glottal),hamming(5),'same'); 
glottal=glottal/sqrt(mean(abs(glottal).^2));

% adult
sig0=(AgP).*(sqrt(1-max(0,min(1,Ag0)).^2).*glottal + 1*max(0,min(1,Ag0)).*noise); 

% Convolve to generate sound
Nf=size(im,1);
time=linspace(1,Nt,Np); 
timel=floor(time); 
timer=time-timel; 
timel1=timel+1; 
timel1(end)=Nt;

if length(sig0)>Fs, % breaks into blocks to avoid memory problems
  sig=zeros(size(sig0'));
  for n1=1:ceil(length(sig0)/Fs),
    idx=max(0,Fs*(n1-1)-Nf+1)+1:min(length(sig0),n1*Fs);
    Sig=sample_window(sig0(idx),Nf,Nf-1,'hanning','same');
    idxvalid=find(idx>Fs*(n1-1)-ceil(Nf/2)+1);
    sig(idx(idxvalid)) = ...
        (1-timer(idx(idxvalid))) .* ...
        sum(Sig(:,idxvalid).*im(:,timel(idx(idxvalid))),1) + ...
        timer(idx(idxvalid)).*sum(Sig(:,idxvalid).*im(:,timel1(idx(idxvalid))),1);
  end
else,
    Sig=sample_window(sig0,Nf,Nf-1,'hanning','same');
    sig=(1-timer).*sum(Sig.*im(:,timel),1) + timer.*sum(Sig.*im(:,timel1),1);
end
%sig=sig*5;

% corrects possible clicks
sig=sig./max(1,convn(abs(sig),30*hamming(51)/sum(hamming(51)),'same')); 

data.sig=sig;
data.Plot.P1=opt.P1;
data.Plot.Af=opt.Af;

